@@ -404,7 +404,7 @@ static void check_is_really_dir(e2fsck_t ctx, struct problem_context *pctx,
 	const char		*old_op;
 	errcode_t		retval;
 	blk_t			blk;
-	int			i, rec_len, not_device = 0;
+	unsigned int		i, rec_len, not_device = 0;
 
 	if (LINUX_S_ISDIR(inode->i_mode) || LINUX_S_ISREG(inode->i_mode) ||
 	    LINUX_S_ISLNK(inode->i_mode) || inode->i_block[0] == 0)
@@ -1655,6 +1655,7 @@ static void scan_extent_node(e2fsck_t ctx, struct problem_context *pctx,
 			problem = PR_1_EXTENT_ENDS_BEYOND;
 
 		if (problem) {
+		report_problem:
 			pctx->blk = extent.e_pblk;
 			pctx->blk2 = extent.e_lblk;
 			pctx->num = extent.e_len;
@@ -1662,11 +1663,7 @@ static void scan_extent_node(e2fsck_t ctx, struct problem_context *pctx,
 				pctx->errcode =
 					ext2fs_extent_delete(ehandle, 0);
 				if (pctx->errcode) {
-					fix_problem(ctx,
-						    PR_1_EXTENT_DELETE_FAIL,
-						    pctx);
-					/* Should never get here */
-					ctx->flags |= E2F_FLAG_ABORT;
+					pctx->str = "ext2fs_extent_delete";
 					return;
 				}
 				pctx->errcode = ext2fs_extent_get(ehandle,
@@ -1682,36 +1679,49 @@ static void scan_extent_node(e2fsck_t ctx, struct problem_context *pctx,
 		}
 
 		if (!is_leaf) {
-			mark_block_used(ctx, extent.e_pblk);
-			pb->num_blocks++;
+			blk = extent.e_pblk;
 			pctx->errcode = ext2fs_extent_get(ehandle,
 						  EXT2_EXTENT_DOWN, &extent);
 			if (pctx->errcode) {
-				printf("Error1: %s on inode %u\n",
-					error_message(pctx->errcode), pctx->ino);
-				abort();
+				pctx->str = "EXT2_EXTENT_DOWN";
+				problem = PR_1_EXTENT_HEADER_INVALID;
+				if (pctx->errcode == EXT2_ET_EXTENT_HEADER_BAD)
+					goto report_problem;
+				return;
 			}
 			scan_extent_node(ctx, pctx, pb, extent.e_lblk, ehandle);
+			if (pctx->errcode)
+				return;
 			pctx->errcode = ext2fs_extent_get(ehandle,
 						  EXT2_EXTENT_UP, &extent);
 			if (pctx->errcode) {
-				printf("Error1: %s on inode %u\n",
-					error_message(pctx->errcode), pctx->ino);
-				abort();
+				pctx->str = "EXT2_EXTENT_UP";
+				return;
 			}
+			mark_block_used(ctx, blk);
+			pb->num_blocks++;
 			goto next;
 		}
 
 		if ((pb->previous_block != 0) &&
 		    (pb->previous_block+1 != extent.e_pblk)) {
-			if (ctx->options & E2F_OPT_FRAGCHECK)
-				printf(("%6lu: expecting %6lu actual extent "
+			if (ctx->options & E2F_OPT_FRAGCHECK) {
+				char type = '?';
+
+				if (pb->is_dir)
+					type = 'd';
+				else if (pb->is_reg)
+					type = 'f';
+
+				printf(("%6lu(%c): expecting %6lu "
+					"actual extent "
 					"phys %6lu log %lu len %lu\n"),
-				       (unsigned long) pctx->ino,
+				       (unsigned long) pctx->ino, type,
 				       (unsigned long) pb->previous_block+1,
 				       (unsigned long) extent.e_pblk,
 				       (unsigned long) extent.e_lblk,
 				       (unsigned long) extent.e_len);
+			}
 			pb->fragmented = 1;
 		}
 		for (blk = extent.e_pblk, blockcnt = extent.e_lblk, i = 0;
@@ -1771,10 +1781,14 @@ static void check_blocks_extents(e2fsck_t ctx, struct problem_context *pctx,
 	}
 
 	scan_extent_node(ctx, pctx, pb, 0, ehandle);
-
-	if (pb->fragmented && pb->num_blocks < fs->super->s_blocks_per_group)
-		ctx->fs_fragmented++;
-
+	if (pctx->errcode &&
+	    fix_problem(ctx, PR_1_EXTENT_ITERATE_FAILURE, pctx)) {
+		pb->num_blocks = 0;
+		inode->i_blocks = 0;
+		e2fsck_clear_inode(ctx, ino, inode, E2F_FLAG_RESTART,
+				   "check_blocks_extents");
+		pctx->errcode = 0;
+	}
 	ext2fs_extent_free(ehandle);
 }
 
@@ -1847,8 +1861,12 @@ static void check_blocks(e2fsck_t ctx, struct problem_context *pctx,
 	if (pctx->errcode)
 		fix_problem(ctx, PR_1_BLOCK_ITERATE, pctx);
 
-	if (pb.fragmented && pb.num_blocks < fs->super->s_blocks_per_group)
-		ctx->fs_fragmented++;
+	if (pb.fragmented && pb.num_blocks < fs->super->s_blocks_per_group) {
+		if (LINUX_S_ISDIR(inode->i_mode))
+			ctx->fs_fragmented_dir++;
+		else
+			ctx->fs_fragmented++;
+	}
 
 	if (pb.clear) {
 		e2fsck_clear_inode(ctx, ino, inode, E2F_FLAG_RESTART,
@@ -1935,7 +1953,7 @@ static void check_blocks(e2fsck_t ctx, struct problem_context *pctx,
 			bad_size = 4;
 		else if ((extent_fs && (inode->i_flags & EXT4_EXTENTS_FL)) &&
 			 size >
-			 ((1LL << (32 + EXT2_BLOCK_SIZE_BITS(fs->super))) - 1))
+			 ((1ULL << (32 + EXT2_BLOCK_SIZE_BITS(fs->super))) - 1))
 			/* too big for an extent-based file - 32bit ee_block */
 			bad_size = 6;
 	}
@@ -2105,12 +2123,21 @@ static int process_block(ext2_filsys fs,
 	 */
 	if (!HOLE_BLKADDR(p->previous_block) && p->ino != EXT2_RESIZE_INO) {
 		if (p->previous_block+1 != blk) {
-			if (ctx->options & E2F_OPT_FRAGCHECK)
-				printf(_("%6lu: expecting %6lu got phys %6lu (blkcnt %lld)\n"),
-				       (unsigned long) pctx->ino,
+			if (ctx->options & E2F_OPT_FRAGCHECK) {
+				char type = '?';
+
+				if (p->is_dir)
+					type = 'd';
+				else if (p->is_reg)
+					type = 'f';
+
+				printf(_("%6lu(%c): expecting %6lu "
+					 "got phys %6lu (blkcnt %lld)\n"),
+				       (unsigned long) pctx->ino, type,
 				       (unsigned long) p->previous_block+1,
 				       (unsigned long) blk,
 				       blockcnt);
+			}
 			p->fragmented = 1;
 		}
 	}
