@@ -26,16 +26,16 @@
 #define PROMPT_RELOCATE	3
 #define PROMPT_ALLOCATE 4
 #define PROMPT_EXPAND	5
-#define PROMPT_CONNECT	6
+#define PROMPT_CONNECT 	6
 #define PROMPT_CREATE	7
 #define PROMPT_SALVAGE	8
 #define PROMPT_TRUNCATE	9
 #define PROMPT_CLEAR_INODE 10
-#define PROMPT_ABORT	11
-#define PROMPT_SPLIT	12
+#define PROMPT_ABORT 	11
+#define PROMPT_SPLIT 	12
 #define PROMPT_CONTINUE	13
 #define PROMPT_CLONE	14
-#define PROMPT_DELETE	15
+#define PROMPT_DELETE 	15
 #define PROMPT_SUPPRESS 16
 #define PROMPT_UNLINK	17
 #define PROMPT_CLEAR_HTREE 18
@@ -240,10 +240,15 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("Clear @j"),
 	  PROMPT_NULL, PR_PREEN_NOMSG },
 
-	/* Filesystem revision is 0, but feature flags are set */
-	{ PR_0_FS_REV_LEVEL,
-	  N_("@f has feature flag(s) set, but is a revision 0 @f.  "),
-	  PROMPT_FIX, PR_PREEN_OK | PR_NO_OK },
+	/* Ask if we should run the journal anyway */
+	{ PR_0_JOURNAL_RUN,
+	  N_("Run @j anyway"),
+	  PROMPT_NULL, 0 },
+
+	/* Run the journal by default */
+	{ PR_0_JOURNAL_RUN_DEFAULT,
+	  N_("Recovery flag not set in backup @S, so running @j anyway.\n"),
+	  PROMPT_NONE, 0 },
 
 	/* Clearing orphan inode */
 	{ PR_0_ORPHAN_CLEAR_INODE,
@@ -252,12 +257,12 @@ static struct e2fsck_problem problem_table[] = {
 
 	/* Illegal block found in orphaned inode */
 	{ PR_0_ORPHAN_ILLEGAL_BLOCK_NUM,
-	   N_("@I %B (%b) found in @o @i %i.\n"),
+	   N_("@I @b #%B (%b) found in @o @i %i.\n"),
 	  PROMPT_NONE, 0 },
 
 	/* Already cleared block found in orphaned inode */
 	{ PR_0_ORPHAN_ALREADY_CLEARED_BLOCK,
-	   N_("Already cleared %B (%b) found in @o @i %i.\n"),
+	   N_("Already cleared @b #%B (%b) found in @o @i %i.\n"),
 	  PROMPT_NONE, 0 },
 
 	/* Illegal orphan inode in superblock */
@@ -270,6 +275,11 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("@I @i %i in @o @i list.\n"),
 	  PROMPT_NONE, 0 },
 
+	/* Filesystem revision is 0, but feature flags are set */
+	{ PR_0_FS_REV_LEVEL,
+	  N_("@f has feature flag(s) set, but is a revision 0 @f.  "),
+	  PROMPT_FIX, PR_PREEN_OK | PR_NO_OK },
+
 	/* Journal superblock has an unknown read-only feature flag set */
 	{ PR_0_JOURNAL_UNSUPP_ROCOMPAT,
 	  N_("@j @S has an unknown read-only feature flag set.\n"),
@@ -301,16 +311,6 @@ static struct e2fsck_problem problem_table[] = {
 	     "Clearing fields beyond the V1 @j @S...\n\n"),
 	  PROMPT_NONE, 0 },
 
-	/* Ask if we should run the journal anyway */
-	{ PR_0_JOURNAL_RUN,
-	  N_("Run @j anyway"),
-	  PROMPT_NULL, 0 },
-
-	/* Run the journal by default */
-	{ PR_0_JOURNAL_RUN_DEFAULT,
-	  N_("Recovery flag not set in backup @S, so running @j anyway.\n"),
-	  PROMPT_NONE, 0 },
-
 	/* Backup journal inode blocks */
 	{ PR_0_BACKUP_JNL,
 	  N_("Backing up @j @i @b information.\n\n"),
@@ -334,13 +334,13 @@ static struct e2fsck_problem problem_table[] = {
 
 	/* Last mount time is in the future */
 	{ PR_0_FUTURE_SB_LAST_MOUNT,
-	  N_("@S last mount time (%t,\n\tnow = %T) is in the future.\n"),
-	  PROMPT_FIX, PR_NO_OK },
+	  N_("@S last mount time is in the future.  "),
+	  PROMPT_FIX, PR_PREEN_OK | PR_NO_OK },
 
 	/* Last write time is in the future */
 	{ PR_0_FUTURE_SB_LAST_WRITE,
-	  N_("@S last write time (%t,\n\tnow = %T) is in the future.\n"),
-	  PROMPT_FIX, PR_NO_OK },
+	  N_("@S last write time is in the future.  "),
+	  PROMPT_FIX, PR_PREEN_OK | PR_NO_OK },
 
 	{ PR_0_EXTERNAL_JOURNAL_HINT,
 	  N_("@S hint for external superblock @s %X.  "),
@@ -354,7 +354,7 @@ static struct e2fsck_problem problem_table[] = {
 	/* group descriptor N checksum is invalid. */
 	{ PR_0_GDT_CSUM,
 	  N_("@g descriptor %g checksum is invalid.  "),
-	     PROMPT_FIX, PR_LATCH_BG_CHECKSUM },
+	     PROMPT_FIX, PR_PREEN_OK },
 
 	/* group descriptor N marked uninitialized without feature set. */
 	{ PR_0_GDT_UNINIT,
@@ -385,23 +385,6 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("The test_fs flag is set (and ext4 is available).  "),
 	  PROMPT_CLEAR, PR_PREEN_OK },
 
-	/* Last mount time is in the future (fudged) */
-	{ PR_0_FUTURE_SB_LAST_MOUNT_FUDGED,
-	  N_("@S last mount time is in the future.\n\t(by less than a day, "
-	     "probably due to the hardware clock being incorrectly set)  "),
-	  PROMPT_FIX, PR_PREEN_OK | PR_NO_OK },
-
-	/* Last write time is in the future (fudged) */
-	{ PR_0_FUTURE_SB_LAST_WRITE_FUDGED,
-	  N_("@S last write time is in the future.\n\t(by less than a day, "
-	     "probably due to the hardware clock being incorrectly set).  "),
-	  PROMPT_FIX, PR_PREEN_OK | PR_NO_OK },
-
-	/* Block group checksum (latch question) is invalid. */
-	{ PR_0_GDT_CSUM_LATCH,
-	  N_("One or more @b @g descriptor checksums are invalid.  "),
-	     PROMPT_FIX, PR_PREEN_OK },
-
 	/* Pass 1 errors */
 
 	/* Pass 1: Checking inodes, blocks, and sizes */
@@ -475,12 +458,12 @@ static struct e2fsck_problem problem_table[] = {
 
 	/* Illegal blocknumber in inode */
 	{ PR_1_ILLEGAL_BLOCK_NUM,
-	  N_("@I %B (%b) in @i %i.  "),
+	  N_("@I @b #%B (%b) in @i %i.  "),
 	  PROMPT_CLEAR, PR_LATCH_BLOCK },
 
 	/* Block number overlaps fs metadata */
 	{ PR_1_BLOCK_OVERLAPS_METADATA,
-	  N_("%B (%b) overlaps @f metadata in @i %i.  "),
+	  N_("@b #%B (%b) overlaps @f metadata in @i %i.  "),
 	  PROMPT_CLEAR, PR_LATCH_BLOCK },
 
 	/* Inode has illegal blocks (latch question) */
@@ -495,7 +478,7 @@ static struct e2fsck_problem problem_table[] = {
 
 	/* Illegal block number in bad block inode */
 	{ PR_1_BB_ILLEGAL_BLOCK_NUM,
-	  N_("@I %B (%b) in bad @b @i.  "),
+	  N_("@I @b #%B (%b) in bad @b @i.  "),
 	  PROMPT_CLEAR, PR_LATCH_BBLOCK },
 
 	/* Bad block inode has illegal blocks (latch question) */
@@ -703,17 +686,17 @@ static struct e2fsck_problem problem_table[] = {
 	/* Error reading Extended Attribute block while fixing refcount */
 	{ PR_1_EXTATTR_READ_ABORT,
 	  N_("Error reading @a @b %b (%m).  "),
-	  PROMPT_NONE, PR_FATAL },
+	  PROMPT_ABORT, 0 },
 
 	/* Extended attribute reference count incorrect */
 	{ PR_1_EXTATTR_REFCOUNT,
-	  N_("@a @b %b has reference count %r, @s %N.  "),
+	  N_("@a @b %b has reference count %B, @s %N.  "),
 	  PROMPT_FIX, 0 },
 
 	/* Error writing Extended Attribute block while fixing refcount */
-	{ PR_1_EXTATTR_WRITE_ABORT,
+	{ PR_1_EXTATTR_WRITE,
 	  N_("Error writing @a @b %b (%m).  "),
-	  PROMPT_NONE, PR_FATAL },
+	  PROMPT_ABORT, 0 },
 
 	/* Multiple EA blocks not supported */
 	{ PR_1_EA_MULTI_BLOCK,
@@ -721,9 +704,9 @@ static struct e2fsck_problem problem_table[] = {
 	  PROMPT_CLEAR, 0},
 
 	/* Error allocating EA region allocation structure */
-	{ PR_1_EA_ALLOC_REGION_ABORT,
+	{ PR_1_EA_ALLOC_REGION,
 	  N_("@A @a @b %b.  "),
-	  PROMPT_NONE, PR_FATAL},
+	  PROMPT_ABORT, 0},
 
 	/* Error EA allocation collision */
 	{ PR_1_EA_ALLOC_COLLISION,
@@ -746,17 +729,17 @@ static struct e2fsck_problem problem_table[] = {
 
 	/* Directory too big */
 	{ PR_1_TOOBIG_DIR,
-	  N_("%B (%b) causes @d to be too big.  "),
+	  N_("@b #%B (%b) causes @d to be too big.  "),
 	  PROMPT_CLEAR, PR_LATCH_TOOBIG },
 
 	/* Regular file too big */
 	{ PR_1_TOOBIG_REG,
-	  N_("%B (%b) causes file to be too big.  "),
+	  N_("@b #%B (%b) causes file to be too big.  "),
 	  PROMPT_CLEAR, PR_LATCH_TOOBIG },
 
 	/* Symlink too big */
 	{ PR_1_TOOBIG_SYMLINK,
-	  N_("%B (%b) causes symlink to be too big.  "),
+	  N_("@b #%B (%b) causes symlink to be too big.  "),
 	  PROMPT_CLEAR, PR_LATCH_TOOBIG },
 
 	/* INDEX_FL flag set on a non-HTREE filesystem */
@@ -798,7 +781,7 @@ static struct e2fsck_problem problem_table[] = {
 	/* Resize inode failed */
 	{ PR_1_RESIZE_INODE_CREATE,
 	  N_("Resize @i (re)creation failed: %m."),
-	  PROMPT_CONTINUE, 0 },
+	  PROMPT_ABORT, 0 },
 
 	/* invalid inode->i_extra_isize */
 	{ PR_1_EXTRA_ISIZE,
@@ -810,6 +793,11 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("@a in @i %i has a namelen (%N) which is @n\n"),
 	  PROMPT_CLEAR, PR_PREEN_OK },
 
+	/* invalid ea entry->e_value_size */
+	{ PR_1_ATTR_VALUE_SIZE,
+	  N_("@a in @i %i has a value size (%N) which is @n\n"),
+	  PROMPT_CLEAR, PR_PREEN_OK },
+
 	/* invalid ea entry->e_value_offs */
 	{ PR_1_ATTR_VALUE_OFFSET,
 	  N_("@a in @i %i has a value offset (%N) which is @n\n"),
@@ -820,11 +808,6 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("@a in @i %i has a value @b (%N) which is @n (must be 0)\n"),
 	  PROMPT_CLEAR, PR_PREEN_OK },
 
-	/* invalid ea entry->e_value_size */
-	{ PR_1_ATTR_VALUE_SIZE,
-	  N_("@a in @i %i has a value size (%N) which is @n\n"),
-	  PROMPT_CLEAR, PR_PREEN_OK },
-
 	/* invalid ea entry->e_hash */
 	{ PR_1_ATTR_HASH,
 	  N_("@a in @i %i has a hash (%N) which is @n\n"),
@@ -885,11 +868,6 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("@i %i has an invalid extent node (blk %b, lblk %c)\n"),
 	  PROMPT_CLEAR, 0 },
 
-	{ PR_1_EOFBLOCKS_FL_SET,
-	  N_("@i %i should not have EOFBLOCKS_FL set "
-	     "(size %Is, lblk %r)\n"),
-	  PROMPT_CLEAR, PR_PREEN_OK },
-
 	/* Pass 1b errors */
 
 	/* Pass 1B: Rescan for duplicate/bad blocks */
@@ -948,7 +926,7 @@ static struct e2fsck_problem problem_table[] = {
 	/* File has duplicate blocks */
 	{ PR_1D_DUP_FILE,
 	  N_("File %Q (@i #%i, mod time %IM) \n"
-	  "  has %r @m @b(s), shared with %N file(s):\n"),
+	  "  has %B @m @b(s), shared with %N file(s):\n"),
 	  PROMPT_NONE, 0 },
 
 	/* List of files sharing duplicate blocks */
@@ -1055,22 +1033,22 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("i_faddr @F %IF, @s zero.\n"),
 	  PROMPT_CLEAR, 0 },
 
-	/* i_file_acl should be zero */
+  	/* i_file_acl should be zero */
 	{ PR_2_FILE_ACL_ZERO,
 	  N_("i_file_acl @F %If, @s zero.\n"),
 	  PROMPT_CLEAR, 0 },
 
-	/* i_dir_acl should be zero */
+  	/* i_dir_acl should be zero */
 	{ PR_2_DIR_ACL_ZERO,
 	  N_("i_dir_acl @F %Id, @s zero.\n"),
 	  PROMPT_CLEAR, 0 },
 
-	/* i_frag should be zero */
+  	/* i_frag should be zero */
 	{ PR_2_FRAG_ZERO,
 	  N_("i_frag @F %N, @s zero.\n"),
 	  PROMPT_CLEAR, 0 },
 
-	/* i_fsize should be zero */
+  	/* i_fsize should be zero */
 	{ PR_2_FSIZE_ZERO,
 	  N_("i_fsize @F %N, @s zero.\n"),
 	  PROMPT_CLEAR, 0 },
@@ -1082,17 +1060,17 @@ static struct e2fsck_problem problem_table[] = {
 
 	/* directory corrupted */
 	{ PR_2_DIR_CORRUPTED,
-	  N_("@d @i %i, %B, offset %N: @d corrupted\n"),
+	  N_("@d @i %i, @b %B, offset %N: @d corrupted\n"),
 	  PROMPT_SALVAGE, 0 },
 
 	/* filename too long */
 	{ PR_2_FILENAME_LONG,
-	  N_("@d @i %i, %B, offset %N: filename too long\n"),
+	  N_("@d @i %i, @b %B, offset %N: filename too long\n"),
 	  PROMPT_TRUNCATE, 0 },
 
 	/* Directory inode has a missing block (hole) */
 	{ PR_2_DIRECTORY_HOLE,
-	  N_("@d @i %i has an unallocated %B.  "),
+	  N_("@d @i %i has an unallocated @b #%B.  "),
 	  PROMPT_ALLOCATE, 0 },
 
 	/* '.' is not NULL terminated */
@@ -1205,7 +1183,7 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("Symlink %Q (@i #%i) is @n.\n"),
 	  PROMPT_CLEAR, 0 },
 
-	/* i_file_acl (extended attribute block) is bad */
+  	/* i_file_acl (extended attribute block) is bad */
 	{ PR_2_FILE_ACL_BAD,
 	  N_("@a @b @F @n (%If).\n"),
 	  PROMPT_CLEAR, 0 },
@@ -1217,22 +1195,22 @@ static struct e2fsck_problem problem_table[] = {
 
 	/* Node in HTREE directory not referenced */
 	{ PR_2_HTREE_NOTREF,
-	  N_("@p @h %d: %B not referenced\n"),
+	  N_("@p @h %d: node (%B) not referenced\n"),
 	  PROMPT_NONE, 0 },
 
 	/* Node in HTREE directory referenced twice */
 	{ PR_2_HTREE_DUPREF,
-	  N_("@p @h %d: %B referenced twice\n"),
+	  N_("@p @h %d: node (%B) referenced twice\n"),
 	  PROMPT_NONE, 0 },
 
 	/* Node in HTREE directory has bad min hash */
 	{ PR_2_HTREE_MIN_HASH,
-	  N_("@p @h %d: %B has bad min hash\n"),
+	  N_("@p @h %d: node (%B) has bad min hash\n"),
 	  PROMPT_NONE, 0 },
 
 	/* Node in HTREE directory has bad max hash */
 	{ PR_2_HTREE_MAX_HASH,
-	  N_("@p @h %d: %B has bad max hash\n"),
+	  N_("@p @h %d: node (%B) has bad max hash\n"),
 	  PROMPT_NONE, 0 },
 
 	/* Clear invalid HTREE directory */
@@ -1256,22 +1234,22 @@ static struct e2fsck_problem problem_table[] = {
 
 	/* Invalid HTREE limit */
 	{ PR_2_HTREE_BAD_LIMIT,
-	  N_("@p @h %d: %B has @n limit (%N)\n"),
+	  N_("@p @h %d: node (%B) has @n limit (%N)\n"),
 	  PROMPT_CLEAR_HTREE, PR_PREEN_OK },
 
 	/* Invalid HTREE count */
 	{ PR_2_HTREE_BAD_COUNT,
-	  N_("@p @h %d: %B has @n count (%N)\n"),
+	  N_("@p @h %d: node (%B) has @n count (%N)\n"),
 	  PROMPT_CLEAR_HTREE, PR_PREEN_OK },
 
 	/* HTREE interior node has out-of-order hashes in table */
 	{ PR_2_HTREE_HASH_ORDER,
-	  N_("@p @h %d: %B has an unordered hash table\n"),
+	  N_("@p @h %d: node (%B) has an unordered hash table\n"),
 	  PROMPT_CLEAR_HTREE, PR_PREEN_OK },
 
 	/* Node in HTREE directory has invalid depth */
 	{ PR_2_HTREE_BAD_DEPTH,
-	  N_("@p @h %d: %B has @n depth (%N)\n"),
+	  N_("@p @h %d: node (%B) has @n depth (%N)\n"),
 	  PROMPT_NONE, 0 },
 
 	/* Duplicate directory entry found */
@@ -1289,7 +1267,7 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("Duplicate @e '%Dn' found.\n\tMarking %p (%i) to be rebuilt.\n\n"),
 	  PROMPT_NONE, 0 },
 
-	/* i_blocks_hi should be zero */
+  	/* i_blocks_hi should be zero */
 	{ PR_2_BLOCKS_HI_ZERO,
 	  N_("i_blocks_hi @F %N, @s zero.\n"),
 	  PROMPT_CLEAR, 0 },
@@ -1308,11 +1286,6 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("@E references @i %Di found in @g %g's unused inodes area.\n"),
 	  PROMPT_FIX, PR_PREEN_OK },
 
-	/* i_blocks_hi should be zero */
-	{ PR_2_I_FILE_ACL_HI_ZERO,
-	  N_("i_file_acl_hi @F %N, @s zero.\n"),
-	  PROMPT_CLEAR, PR_PREEN_OK },
-
 	/* Pass 3 errors */
 
 	/* Pass 3: Checking directory connectivity */
@@ -1619,6 +1592,11 @@ static struct e2fsck_problem problem_table[] = {
 	  " +(%i--%j)",
 	  PROMPT_NONE, PR_LATCH_IBITMAP | PR_PREEN_OK | PR_PREEN_NOMSG },
 
+	/* Recreate journal if E2F_FLAG_JOURNAL_INODE flag is set */
+	{ PR_6_RECREATE_JOURNAL,
+	  N_("Recreate @j"),
+	  PROMPT_NULL, PR_PREEN_OK | PR_NO_OK },
+
 	/* Group N block(s) in use but group is marked BLOCK_UNINIT */
 	{ PR_5_BLOCK_UNINIT,
 	  N_("@g %g @b(s) in use but @g is marked BLOCK_UNINIT\n"),
@@ -1629,13 +1607,6 @@ static struct e2fsck_problem problem_table[] = {
 	  N_("@g %g @i(s) in use but @g is marked INODE_UNINIT\n"),
 	  PROMPT_FIX, PR_PREEN_OK },
 
-	/* Post-Pass 5 errors */
-
-	/* Recreate journal if E2F_FLAG_JOURNAL_INODE flag is set */
-	{ PR_6_RECREATE_JOURNAL,
-	  N_("Recreate @j"),
-	  PROMPT_NULL, PR_PREEN_OK | PR_NO_OK },
-
 	{ 0 }
 };
 
@@ -1655,13 +1626,12 @@ static struct latch_descr pr_latch_info[] = {
 	{ PR_LATCH_LOW_DTIME, PR_1_ORPHAN_LIST_REFUGEES, 0 },
 	{ PR_LATCH_TOOBIG, PR_1_INODE_TOOBIG, 0 },
 	{ PR_LATCH_OPTIMIZE_DIR, PR_3A_OPTIMIZE_DIR_HEADER, PR_3A_OPTIMIZE_DIR_END },
-	{ PR_LATCH_BG_CHECKSUM, PR_0_GDT_CSUM_LATCH, 0 },
 	{ -1, 0, 0 },
 };
 
 static struct e2fsck_problem *find_problem(problem_t code)
 {
-	int	i;
+	int 	i;
 
 	for (i=0; problem_table[i].e2p_code; i++) {
 		if (problem_table[i].e2p_code == code)
@@ -1746,7 +1716,7 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 	struct e2fsck_problem *ptr;
 	struct latch_descr *ldesc = 0;
 	const char *message;
-	int		def_yn, answer, ans;
+	int 		def_yn, answer, ans;
 	int		print_answer = 0;
 	int		suppress = 0;
 
@@ -1773,7 +1743,6 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 		reconfigure_bool(ctx, ptr, key, PR_NOCOLLATE, "no_collate");
 		reconfigure_bool(ctx, ptr, key, PR_NO_NOMSG, "no_nomsg");
 		reconfigure_bool(ctx, ptr, key, PR_PREEN_NOHDR, "preen_noheader");
-		reconfigure_bool(ctx, ptr, key, PR_FORCE_NO, "force_no");
 
 		ptr->flags |= PR_CONFIG;
 	}
@@ -1804,7 +1773,7 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 	    (ctx->options & E2F_OPT_PREEN))
 		suppress++;
 	if ((ptr->flags & PR_NO_NOMSG) &&
-	    ((ctx->options & E2F_OPT_NO) || (ptr->flags & PR_FORCE_NO)))
+	    (ctx->options & E2F_OPT_NO))
 		suppress++;
 	if (!suppress) {
 		message = ptr->e2p_description;
@@ -1828,11 +1797,7 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 		else
 			answer = def_yn;
 	} else {
-		if (ptr->flags & PR_FORCE_NO) {
-			answer = 0;
-			if (!suppress)
-				print_answer = 1;
-		} else if (ctx->options & E2F_OPT_PREEN) {
+		if (ctx->options & E2F_OPT_PREEN) {
 			answer = def_yn;
 			if (!(ptr->flags & PR_PREEN_NOMSG))
 				print_answer = 1;
@@ -1864,89 +1829,3 @@ int fix_problem(e2fsck_t ctx, problem_t code, struct problem_context *pctx)
 
 	return answer;
 }
-
-#ifdef UNITTEST
-
-#include <stdlib.h>
-#include <stdio.h>
-
-errcode_t
-profile_get_boolean(profile_t profile, const char *name, const char *subname,
-		    const char *subsubname, int def_val, int *ret_boolean)
-{
-	return 0;
-}
-
-void print_e2fsck_message(e2fsck_t ctx, const char *msg,
-			  struct problem_context *pctx, int first,
-			  int recurse)
-{
-	return;
-}
-
-void fatal_error(e2fsck_t ctx, const char *msg)
-{
-	return;
-}
-
-void preenhalt(e2fsck_t ctx)
-{
-	return;
-}
-
-errcode_t
-profile_get_string(profile_t profile, const char *name, const char *subname,
-		   const char *subsubname, const char *def_val,
-		   char **ret_string)
-{
-	return 0;
-}
-
-int ask (e2fsck_t ctx, const char * string, int def)
-{
-	return 0;
-}
-
-int verify_problem_table(e2fsck_t ctx)
-{
-	struct e2fsck_problem *curr, *prev = NULL;
-	int rc = 0;
-
-	for (prev = NULL, curr = problem_table; curr->e2p_code; prev = curr++) {
-		if (prev == NULL)
-			continue;
-
-		if (curr->e2p_code > prev->e2p_code)
-			continue;
-
-		if (curr->e2p_code == prev->e2p_code)
-			fprintf(stderr, "*** Duplicate in problem table:\n");
-		else
-			fprintf(stderr, "*** Unordered problem table:\n");
-
-		fprintf(stderr, "curr code = 0x%08x: %s\n",
-			curr->e2p_code, curr->e2p_description);
-		fprintf(stderr, "*** prev code = 0x%08x: %s\n",
-			prev->e2p_code, prev->e2p_description);
-
-		fprintf(stderr, "*** This is a %sprogramming error in e2fsck\n",
-			(curr->e2p_code == prev->e2p_code) ? "fatal " : "");
-
-		rc = 1;
-	}
-
-	return rc;
-}
-
-int main(int argc, char *argv[])
-{
-	e2fsck_t ctx;
-	int rc;
-
-	rc = verify_problem_table(ctx);
-	if (rc == 0)
-		printf("e2fsck problem table verified\n");
-
-	return rc;
-}
-#endif /* UNITTEST */
