@@ -1,13 +1,13 @@
 /*
  * rehash.c --- rebuild hash tree directories
- * 
+ *
  * Copyright (C) 2002 Theodore Ts'o
  *
  * %Begin-Header%
  * This file may be redistributed under the terms of the GNU Public
  * License.
  * %End-Header%
- * 
+ *
  * This algorithm is designed for simplicity of implementation and to
  * pack the directory as much as possible.  It however requires twice
  * as much memory as the size of the directory.  The maximum size
@@ -36,7 +36,7 @@
  *    --------------------------------------------------------
  *                  ^ ptr    ^ptr
  *                tail new   head old
- * 
+ *
  * This is going to be a pain in the tuckus to implement, and will
  * require a lot more disk accesses.  So I'm going to skip it for now;
  * it's only really going to be an issue for really, really big
@@ -89,8 +89,8 @@ static int fill_dir_block(ext2_filsys fs,
 	struct ext2_dir_entry 	*dirent;
 	char			*dir;
 	unsigned int		offset, dir_offset;
-	int			hash_alg;
-	
+	int			rec_len, hash_alg;
+
 	if (blockcnt < 0)
 		return 0;
 
@@ -117,14 +117,16 @@ static int fill_dir_block(ext2_filsys fs,
 	dir_offset = 0;
 	while (dir_offset < fs->blocksize) {
 		dirent = (struct ext2_dir_entry *) (dir + dir_offset);
-		if (((dir_offset + dirent->rec_len) > fs->blocksize) ||
-		    (dirent->rec_len < 8) ||
-		    ((dirent->rec_len % 4) != 0) ||
-		    (((dirent->name_len & 0xFF)+8) > dirent->rec_len)) {
+		rec_len = (dirent->rec_len || fs->blocksize < 65536) ?
+			dirent->rec_len : 65536;
+		if (((dir_offset + rec_len) > fs->blocksize) ||
+		    (rec_len < 8) ||
+		    ((rec_len % 4) != 0) ||
+		    (((dirent->name_len & 0xFF)+8) > rec_len)) {
 			fd->err = EXT2_ET_DIR_CORRUPTED;
 			return BLOCK_ABORT;
 		}
-		dir_offset += dirent->rec_len;
+		dir_offset += rec_len;
 		if (dirent->inode == 0)
 			continue;
 		if (!fd->compress && ((dirent->name_len&0xFF) == 1) &&
@@ -160,7 +162,7 @@ static int fill_dir_block(ext2_filsys fs,
 				return BLOCK_ABORT;
 		}
 	}
-	
+
 	return 0;
 }
 
@@ -203,7 +205,7 @@ static EXT2_QSORT_TYPE hash_cmp(const void *a, const void *b)
 	const struct hash_entry *he_a = (const struct hash_entry *) a;
 	const struct hash_entry *he_b = (const struct hash_entry *) b;
 	int	ret;
-	
+
 	if (he_a->hash > he_b->hash)
 		ret = 1;
 	else if (he_a->hash < he_b->hash)
@@ -219,7 +221,7 @@ static EXT2_QSORT_TYPE hash_cmp(const void *a, const void *b)
 	return ret;
 }
 
-static errcode_t alloc_size_dir(ext2_filsys fs, struct out_dir *outdir, 
+static errcode_t alloc_size_dir(ext2_filsys fs, struct out_dir *outdir,
 				int blocks)
 {
 	void			*new_mem;
@@ -278,7 +280,7 @@ static void mutate_name(char *str, __u16 *len)
 {
 	int	i;
 	__u16	l = *len & 0xFF, h = *len & 0xff00;
-	
+
 	/*
 	 * First check to see if it looks the name has been mutated
 	 * already
@@ -321,7 +323,7 @@ static void mutate_name(char *str, __u16 *len)
 		} else {
 			if (str[0] == '~')
 				str[0] = 'a';
-			else 
+			else
 				str[0]++;
 		}
 		break;
@@ -339,7 +341,7 @@ static int duplicate_search_and_fix(e2fsck_t ctx, ext2_filsys fs,
 	char			new_name[256];
 	__u16			new_len;
 	int			hash_alg;
-	
+
 	clear_problem_context(&pctx);
 	pctx.ino = ino;
 
@@ -376,7 +378,7 @@ static int duplicate_search_and_fix(e2fsck_t ctx, ext2_filsys fs,
 				     new_len & 0xFF)))
 				continue;
 			mutate_name(new_name, &new_len);
-			
+
 			j = -1;
 		}
 		new_name[new_len & 0xFF] = 0;
@@ -395,18 +397,28 @@ static int duplicate_search_and_fix(e2fsck_t ctx, ext2_filsys fs,
 }
 
 
-static errcode_t copy_dir_entries(ext2_filsys fs,
+static errcode_t copy_dir_entries(e2fsck_t ctx,
 				  struct fill_dir_struct *fd,
 				  struct out_dir *outdir)
 {
+	ext2_filsys 		fs = ctx->fs;
 	errcode_t		retval;
 	char			*block_start;
 	struct hash_entry 	*ent;
 	struct ext2_dir_entry	*dirent;
 	int			i, rec_len, left;
 	ext2_dirhash_t		prev_hash;
-	int			offset;
-	
+	int			offset, slack;
+
+	if (ctx->htree_slack_percentage == 255) {
+		profile_get_uint(ctx->profile, "options",
+				 "indexed_dir_slack_percentage",
+				 0, 20,
+				 &ctx->htree_slack_percentage);
+		if (ctx->htree_slack_percentage > 100)
+			ctx->htree_slack_percentage = 20;
+	}
+
 	outdir->max = 0;
 	retval = alloc_size_dir(fs, outdir,
 				(fd->dir_size / fs->blocksize) + 2);
@@ -420,6 +432,10 @@ static errcode_t copy_dir_entries(ext2_filsys fs,
 		return retval;
 	dirent = (struct ext2_dir_entry *) block_start;
 	left = fs->blocksize;
+	slack = fd->compress ? 12 :
+		(fs->blocksize * ctx->htree_slack_percentage)/100;
+	if (slack < 12)
+		slack = 12;
 	for (i=0; i < fd->num_array; i++) {
 		ent = fd->harray + i;
 		if (ent->dir->inode == 0)
@@ -447,7 +463,7 @@ static errcode_t copy_dir_entries(ext2_filsys fs,
 		memcpy(dirent->name, ent->dir->name, dirent->name_len & 0xFF);
 		offset += rec_len;
 		left -= rec_len;
-		if (left < 12) {
+		if (left < slack) {
 			dirent->rec_len += left;
 			offset += left;
 			left = 0;
@@ -471,7 +487,7 @@ static struct ext2_dx_root_info *set_root_node(ext2_filsys fs, char *buf,
 
 	if (fs->super->s_feature_incompat & EXT2_FEATURE_INCOMPAT_FILETYPE)
 		filetype = EXT2_FT_DIR << 8;
-	
+
 	memset(buf, 0, fs->blocksize);
 	dir = (struct ext2_dir_entry *) buf;
 	dir->inode = ino;
@@ -484,7 +500,7 @@ static struct ext2_dx_root_info *set_root_node(ext2_filsys fs, char *buf,
 	dir->name[1] = '.';
 	dir->name_len = 2 | filetype;
 	dir->rec_len = fs->blocksize - 12;
-	
+
 	root = (struct ext2_dx_root_info *) (buf+24);
 	root->reserved_zero = 0;
 	root->hash_version = fs->super->s_def_hash_version;
@@ -509,7 +525,7 @@ static struct ext2_dx_entry *set_int_node(ext2_filsys fs, char *buf)
 	dir = (struct ext2_dir_entry *) buf;
 	dir->inode = 0;
 	dir->rec_len = fs->blocksize;
-	
+
 	limits = (struct ext2_dx_countlimit *) (buf+8);
 	limits->limit = (fs->blocksize - 8) / sizeof(struct ext2_dx_entry);
 	limits->count = 0;
@@ -533,7 +549,7 @@ static errcode_t calculate_tree(ext2_filsys fs,
 	char				* block_start;
 	int				i, c1, c2, nblks;
 	int				limit_offset, root_offset;
-	
+
 	root_info = set_root_node(fs, outdir->buf, ino, parent);
 	root_offset = limit_offset = ((char *) root_info - outdir->buf) +
 		root_info->info_length;
@@ -562,7 +578,7 @@ static errcode_t calculate_tree(ext2_filsys fs,
 				return ENOSPC;
 			if (c2 == 0) {
 				if (limit)
-					limit->limit = limit->count = 
+					limit->limit = limit->count =
 		ext2fs_cpu_to_le16(limit->limit);
 				root = (struct ext2_dx_entry *)
 					(outdir->buf + root_offset);
@@ -610,7 +626,7 @@ static int write_dir_block(ext2_filsys fs,
 			   blk_t	*block_nr,
 			   e2_blkcnt_t blockcnt,
 			   blk_t ref_block EXT2FS_ATTR((unused)),
-			   int ref_offset EXT2FS_ATTR((unused)), 
+			   int ref_offset EXT2FS_ATTR((unused)),
 			   void *priv_data)
 {
 	struct write_dir_struct	*wd = (struct write_dir_struct *) priv_data;
@@ -682,7 +698,7 @@ errcode_t e2fsck_rehash_dir(e2fsck_t ctx, ext2_ino_t ino)
 	char			*dir_buf = 0;
 	struct fill_dir_struct	fd;
 	struct out_dir		outdir;
-	
+
 	outdir.max = outdir.num = 0;
 	outdir.buf = 0;
 	outdir.hashes = 0;
@@ -748,10 +764,10 @@ resort:
 	 * Copy the directory entries.  In a htree directory these
 	 * will become the leaf nodes.
 	 */
-	retval = copy_dir_entries(fs, &fd, &outdir);
+	retval = copy_dir_entries(ctx, &fd, &outdir);
 	if (retval)
 		goto errout;
-	
+
 	free(dir_buf); dir_buf = 0;
 
 	if (!fd.compress) {
@@ -760,7 +776,7 @@ resort:
 		if (retval)
 			goto errout;
 	}
-	
+
 	retval = write_directory(ctx, fs, &outdir, ino, fd.compress);
 	if (retval)
 		goto errout;
@@ -798,7 +814,7 @@ void e2fsck_rehash_directories(e2fsck_t ctx)
 		return;
 
 	e2fsck_get_lost_and_found(ctx, 0);
-		
+
 	clear_problem_context(&pctx);
 
 	dir_index = ctx->fs->super->s_feature_compat & EXT2_FEATURE_COMPAT_DIR_INDEX;
@@ -807,7 +823,7 @@ void e2fsck_rehash_directories(e2fsck_t ctx)
 		dirinfo_iter = e2fsck_dir_info_iter_begin(ctx);
 		max = e2fsck_get_num_dirinfo(ctx);
 	} else {
-		retval = ext2fs_u32_list_iterate_begin(ctx->dirs_to_hash, 
+		retval = ext2fs_u32_list_iterate_begin(ctx->dirs_to_hash,
 						       &iter);
 		if (retval) {
 			pctx.errcode = retval;
@@ -818,7 +834,7 @@ void e2fsck_rehash_directories(e2fsck_t ctx)
 	}
 	while (1) {
 		if (all_dirs) {
-			if ((dir = e2fsck_dir_info_iter(ctx, 
+			if ((dir = e2fsck_dir_info_iter(ctx,
 							dirinfo_iter)) == 0)
 				break;
 			ino = dir->ino;
@@ -850,7 +866,7 @@ void e2fsck_rehash_directories(e2fsck_t ctx)
 		e2fsck_dir_info_iter_end(ctx, dirinfo_iter);
 	else
 		ext2fs_u32_list_iterate_end(iter);
-	
+
 	if (ctx->dirs_to_hash)
 		ext2fs_u32_list_free(ctx->dirs_to_hash);
 	ctx->dirs_to_hash = 0;
