@@ -304,7 +304,7 @@ return_abort:
  * resize inode should be cleared) as well as the case where on-line
  * resizing is enabled.
  */
-void check_resize_inode(e2fsck_t ctx)
+static void check_resize_inode(e2fsck_t ctx)
 {
 	ext2_filsys fs = ctx->fs;
 	struct ext2_inode inode;
@@ -457,14 +457,12 @@ void check_super_block(e2fsck_t ctx)
 	blk_t	first_block, last_block;
 	struct ext2_super_block *sb = fs->super;
 	struct ext2_group_desc *gd;
-	problem_t	problem;
 	blk_t	blocks_per_group = fs->super->s_blocks_per_group;
 	blk_t	bpg_max;
 	int	inodes_per_block;
 	int	ipg_max;
 	int	inode_size;
-	int	accept_time_fudge;
-	int	broken_system_clock;
+	int	buggy_init_scripts;
 	dgrp_t	i;
 	blk_t	should_be;
 	struct problem_context	pctx;
@@ -787,6 +785,8 @@ void check_super_block(e2fsck_t ctx)
 		fs->flags &= ~EXT2_FLAG_MASTER_SB_ONLY;
 	}
 
+	check_resize_inode(ctx);
+
 	/*
 	 * Clean up any orphan inodes, if present.
 	 */
@@ -796,59 +796,40 @@ void check_super_block(e2fsck_t ctx)
 	}
 
 	/*
-	 * Unfortunately, due to Windows' unfortunate design decision
-	 * to configure the hardware clock to tick localtime, instead
-	 * of the more proper and less error-prone UTC time, many
-	 * users end up in the situation where the system clock is
-	 * incorrectly set at the time when e2fsck is run.
-	 *
-	 * Historically this was usually due to some distributions
-	 * having buggy init scripts and/or installers that didn't
-	 * correctly detect this case and take appropriate
-	 * countermeasures.  However, it's still possible, despite the
-	 * best efforts of init script and installer authors to not be
-	 * able to detect this misconfiguration, usually due to a
-	 * buggy or misconfigured virtualization manager or the
-	 * installer not having access to a network time server during
-	 * the installation process.  So by default, we allow the
-	 * superblock times to be fudged by up to 24 hours.  This can
-	 * be disabled by setting options.accept_time_fudge to the
-	 * boolean value of false in e2fsck.conf.  We also support
-	 * options.buggy_init_scripts for backwards compatibility.
+	 * Some buggy distributions (such as Ubuntu) have init scripts
+	 * and/or installers which fail to correctly set the system
+	 * clock before running e2fsck and/or formatting the
+	 * filesystem initially.  Normally this happens because the
+	 * hardware clock is ticking localtime, instead of the more
+	 * proper and less error-prone UTC time.  So while the kernel
+	 * is booting, the system time (which in Linux systems always
+	 * ticks in UTC time) is set from the hardware clock, but
+	 * since the hardware clock is ticking localtime, the system
+	 * time is incorrect.  Unfortunately, some buggy distributions
+	 * do not correct this before running e2fsck.  If this option
+	 * is set to a boolean value of true, we attempt to work
+	 * around this situation by allowing the superblock last write
+	 * time, last mount time, and last check time to be in the
+	 * future by up to 24 hours.
 	 */
-	profile_get_boolean(ctx->profile, "options", "accept_time_fudge",
-			    0, 1, &accept_time_fudge);
 	profile_get_boolean(ctx->profile, "options", "buggy_init_scripts",
-			    0, accept_time_fudge, &accept_time_fudge);
-	ctx->time_fudge = accept_time_fudge ? 86400 : 0;
-
-	profile_get_boolean(ctx->profile, "options", "broken_system_clock",
-			    0, 0, &broken_system_clock);
+			    0, 0, &buggy_init_scripts);
+	ctx->time_fudge = buggy_init_scripts ? 86400 : 0;
 
 	/*
 	 * Check to see if the superblock last mount time or last
 	 * write time is in the future.
 	 */
-	if (!broken_system_clock &&
-	    !(ctx->flags & E2F_FLAG_TIME_INSANE) &&
-	    fs->super->s_mtime > (__u32) ctx->now) {
+	if (fs->super->s_mtime > (__u32) ctx->now + ctx->time_fudge) {
 		pctx.num = fs->super->s_mtime;
-		problem = PR_0_FUTURE_SB_LAST_MOUNT;
-		if (fs->super->s_mtime <= (__u32) ctx->now + ctx->time_fudge)
-			problem = PR_0_FUTURE_SB_LAST_MOUNT_FUDGED;
-		if (fix_problem(ctx, problem, &pctx)) {
+		if (fix_problem(ctx, PR_0_FUTURE_SB_LAST_MOUNT, &pctx)) {
 			fs->super->s_mtime = ctx->now;
 			ext2fs_mark_super_dirty(fs);
 		}
 	}
-	if (!broken_system_clock &&
-	    !(ctx->flags & E2F_FLAG_TIME_INSANE) &&
-	    fs->super->s_wtime > (__u32) ctx->now) {
+	if (fs->super->s_wtime > (__u32) ctx->now + ctx->time_fudge) {
 		pctx.num = fs->super->s_wtime;
-		problem = PR_0_FUTURE_SB_LAST_WRITE;
-		if (fs->super->s_wtime <= (__u32) ctx->now + ctx->time_fudge)
-			problem = PR_0_FUTURE_SB_LAST_WRITE_FUDGED;
-		if (fix_problem(ctx, problem, &pctx)) {
+		if (fix_problem(ctx, PR_0_FUTURE_SB_LAST_WRITE, &pctx)) {
 			fs->super->s_wtime = ctx->now;
 			ext2fs_mark_super_dirty(fs);
 		}
@@ -883,8 +864,7 @@ void check_super_block(e2fsck_t ctx)
  * try to discourage it in the future.  In particular, for the newer
  * ext4 files, especially EXT4_FEATURE_RO_COMPAT_DIR_NLINK and
  * EXT3_FEATURE_INCOMPAT_EXTENTS.  So some of these may go away in the
- * future.  EXT3_FEATURE_INCOMPAT_RECOVER may also get set when
- * copying the primary superblock during online resize.
+ * future.
  *
  * The kernel will set EXT2_FEATURE_COMPAT_EXT_ATTR, but
  * unfortunately, we shouldn't ignore it since if it's not set in the
@@ -893,18 +873,16 @@ void check_super_block(e2fsck_t ctx)
  */
 #define FEATURE_RO_COMPAT_IGNORE	(EXT2_FEATURE_RO_COMPAT_LARGE_FILE| \
 					 EXT4_FEATURE_RO_COMPAT_DIR_NLINK)
-#define FEATURE_INCOMPAT_IGNORE		(EXT3_FEATURE_INCOMPAT_EXTENTS| \
-					 EXT3_FEATURE_INCOMPAT_RECOVER)
+#define FEATURE_INCOMPAT_IGNORE		(EXT3_FEATURE_INCOMPAT_EXTENTS)
 
 int check_backup_super_block(e2fsck_t ctx)
 {
 	ext2_filsys	fs = ctx->fs;
+	ext2_filsys	tfs = 0;
 	errcode_t	retval;
 	dgrp_t		g;
 	blk_t		sb;
 	int		ret = 0;
-	char		buf[SUPERBLOCK_SIZE];
-	struct ext2_super_block	*backup_sb;
 
 	/*
 	 * If we are already writing out the backup blocks, then we
@@ -927,38 +905,32 @@ int check_backup_super_block(e2fsck_t ctx)
 		sb = fs->super->s_first_data_block +
 			(g * fs->super->s_blocks_per_group);
 
-		retval = io_channel_read_blk(fs->io, sb, -SUPERBLOCK_SIZE,
-					     buf);
-		if (retval)
-			continue;
-		backup_sb = (struct ext2_super_block *) buf;
-#ifdef WORDS_BIGENDIAN
-		ext2fs_swap_super(backup_sb);
-#endif
-		if ((backup_sb->s_magic != EXT2_SUPER_MAGIC) ||
-		    (backup_sb->s_rev_level > EXT2_LIB_CURRENT_REV) ||
-		    ((backup_sb->s_log_block_size + EXT2_MIN_BLOCK_LOG_SIZE) >
-		     EXT2_MAX_BLOCK_LOG_SIZE) ||
-		    (EXT2_INODE_SIZE(backup_sb) < EXT2_GOOD_OLD_INODE_SIZE))
+		retval = ext2fs_open(ctx->filesystem_name, 0,
+				     sb, fs->blocksize,
+				     fs->io->manager, &tfs);
+		if (retval) {
+			tfs = 0;
 			continue;
+		}
 
 #define SUPER_INCOMPAT_DIFFERENT(x)	\
-	((fs->super->x & ~FEATURE_INCOMPAT_IGNORE) !=	\
-	 (backup_sb->x & ~FEATURE_INCOMPAT_IGNORE))
+	(( fs->super->x & ~FEATURE_INCOMPAT_IGNORE) !=	\
+	 (tfs->super->x & ~FEATURE_INCOMPAT_IGNORE))
 #define SUPER_RO_COMPAT_DIFFERENT(x)	\
-	((fs->super->x & ~FEATURE_RO_COMPAT_IGNORE) !=	\
-	 (backup_sb->x & ~FEATURE_RO_COMPAT_IGNORE))
+	(( fs->super->x & ~FEATURE_RO_COMPAT_IGNORE) !=	\
+	 (tfs->super->x & ~FEATURE_RO_COMPAT_IGNORE))
 #define SUPER_DIFFERENT(x)		\
-	(fs->super->x != backup_sb->x)
+	(fs->super->x != tfs->super->x)
 
 		if (SUPER_DIFFERENT(s_feature_compat) ||
 		    SUPER_INCOMPAT_DIFFERENT(s_feature_incompat) ||
 		    SUPER_RO_COMPAT_DIFFERENT(s_feature_ro_compat) ||
 		    SUPER_DIFFERENT(s_blocks_count) ||
 		    SUPER_DIFFERENT(s_inodes_count) ||
-		    memcmp(fs->super->s_uuid, backup_sb->s_uuid,
+		    memcmp(fs->super->s_uuid, tfs->super->s_uuid,
 			   sizeof(fs->super->s_uuid)))
 			ret = 1;
+		ext2fs_close(tfs);
 		break;
 	}
 	return ret;
