@@ -4,8 +4,8 @@
  * Copyright (C) 1993, 1994, 1994, 1995, 1996, 1997 Theodore Ts'o.
  *
  * %Begin-Header%
- * This file may be redistributed under the terms of the GNU Library
- * General Public License, version 2.
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
  * %End-Header%
  */
 
@@ -21,43 +21,6 @@
 #include "ext2_fs.h"
 #include "ext2fsP.h"
 
-#define EXT4_MAX_REC_LEN		((1<<16)-1)
-
-errcode_t ext2fs_get_rec_len(ext2_filsys fs,
-			     struct ext2_dir_entry *dirent,
-			     unsigned int *rec_len)
-{
-	unsigned int len = dirent->rec_len;
-
-	if (fs->blocksize < 65536)
-		*rec_len = len;
-	else if (len == EXT4_MAX_REC_LEN || len == 0)
-		*rec_len = fs->blocksize;
-	else 
-		*rec_len = (len & 65532) | ((len & 3) << 16);
-	return 0;
-}
-
-errcode_t ext2fs_set_rec_len(ext2_filsys fs,
-			     unsigned int len,
-			     struct ext2_dir_entry *dirent)
-{
-	if ((len > fs->blocksize) || (fs->blocksize > (1 << 18)) || (len & 3))
-		return EINVAL;
-	if (len < 65536) {
-		dirent->rec_len = len;
-		return 0;
-	}
-	if (len == fs->blocksize) {
-		if (fs->blocksize == 65536)
-			dirent->rec_len = EXT4_MAX_REC_LEN;
-		else 
-			dirent->rec_len = 0;
-	} else
-		dirent->rec_len = (len & 65532) | ((len >> 16) & 3);
-	return 0;
-}
-
 /*
  * This function checks to see whether or not a potential deleted
  * directory entry looks valid.  What we do is check the deleted entry
@@ -66,23 +29,20 @@ errcode_t ext2fs_set_rec_len(ext2_filsys fs,
  * undeleted entry.  Returns 1 if the deleted entry looks valid, zero
  * if not valid.
  */
-static int ext2fs_validate_entry(ext2_filsys fs, char *buf,
-				 unsigned int offset,
-				 unsigned int final_offset)
+static int ext2fs_validate_entry(ext2_filsys fs, char *buf, int offset,
+				 int final_offset)
 {
 	struct ext2_dir_entry *dirent;
-	unsigned int rec_len;
-#define DIRENT_MIN_LENGTH 12
+	int	rec_len;
 
-	while ((offset < final_offset) &&
-	       (offset <= fs->blocksize - DIRENT_MIN_LENGTH)) {
+	while (offset < final_offset) {
 		dirent = (struct ext2_dir_entry *)(buf + offset);
-		if (ext2fs_get_rec_len(fs, dirent, &rec_len))
-			return 0;
+		rec_len = (dirent->rec_len || fs->blocksize < 65536) ?
+			dirent->rec_len : 65536;
 		offset += rec_len;
 		if ((rec_len < 8) ||
 		    ((rec_len % 4) != 0) ||
-		    ((((unsigned) dirent->name_len & 0xFF)+8) > rec_len))
+		    (((dirent->name_len & 0xFF)+8) > rec_len))
 			return 0;
 	}
 	return (offset == final_offset);
@@ -188,8 +148,7 @@ int ext2fs_process_dir_block(ext2_filsys fs,
 	int		ret = 0;
 	int		changed = 0;
 	int		do_abort = 0;
-	unsigned int	rec_len, size;
-	int		entry;
+	int		rec_len, entry, size;
 	struct ext2_dir_entry *dirent;
 
 	if (blockcnt < 0)
@@ -203,12 +162,12 @@ int ext2fs_process_dir_block(ext2_filsys fs,
 
 	while (offset < fs->blocksize) {
 		dirent = (struct ext2_dir_entry *) (ctx->buf + offset);
-		if (ext2fs_get_rec_len(fs, dirent, &rec_len))
-			return BLOCK_ABORT;
+		rec_len = (dirent->rec_len || fs->blocksize < 65536) ?
+			dirent->rec_len : 65536;
 		if (((offset + rec_len) > fs->blocksize) ||
 		    (rec_len < 8) ||
 		    ((rec_len % 4) != 0) ||
-		    ((((unsigned) dirent->name_len & 0xFF)+8) > rec_len)) {
+		    (((dirent->name_len & 0xFF)+8) > rec_len)) {
 			ctx->errcode = EXT2_ET_DIR_CORRUPTED;
 			return BLOCK_ABORT;
 		}
@@ -226,8 +185,8 @@ int ext2fs_process_dir_block(ext2_filsys fs,
 			entry++;
 
 		if (ret & DIRENT_CHANGED) {
-			if (ext2fs_get_rec_len(fs, dirent, &rec_len))
-				return BLOCK_ABORT;
+			rec_len = (dirent->rec_len || fs->blocksize < 65536) ?
+				dirent->rec_len : 65536;
 			changed++;
 		}
 		if (ret & DIRENT_ABORT) {
