@@ -4,11 +4,8 @@
  * Copyright (c) 1997 by Theodore Ts'o.
  *
  * Copyright (c) 1997 Mark Habersack
+ * This file may be distributed under the terms of the GNU Public License.
  *
- * %Begin-Header%
- * This file may be redistributed under the terms of the GNU Library
- * General Public License, version 2.
- * %End-Header%
  */
 
 #include <stdio.h>
@@ -281,8 +278,8 @@ static errcode_t dos_open(const char *dev, int flags, io_channel *channel)
   if(!HW_OK())
   {
     _dio_error = ERR_HARDWARE;
-    free(part->dev);
-    free(part);
+    if (part)
+	    free(part);
     return EFAULT;
   }
 
@@ -301,8 +298,8 @@ static errcode_t dos_open(const char *dev, int flags, io_channel *channel)
   if(!HW_OK())
   {
     _dio_error = ERR_HARDWARE;
-    free(part->dev);
-    free(part);
+    if (part)
+	    free(part);
     return EFAULT;
   }
 
@@ -313,8 +310,8 @@ static errcode_t dos_open(const char *dev, int flags, io_channel *channel)
   {
     _dio_error = part->pno == 0xFE ? ERR_EMPTYPART :
                  part->pno == 0xFD ? ERR_LINUXSWAP : ERR_NOTEXT2FS;
-    free(part->dev);
-    free(part);
+    if (part)
+	    free(part);
     return ENODEV;
   }
 
@@ -355,8 +352,10 @@ static errcode_t dos_open(const char *dev, int flags, io_channel *channel)
 
 static errcode_t dos_close(io_channel channel)
 {
-	free(channel->name);
-	free(channel);
+	if (channel->name)
+		free(channel->name);
+	if (channel)
+		free(channel);
 
 	return 0;
 }
