@@ -50,8 +50,8 @@ int hex_format = 0;
 
 static void usage(void)
 {
-	fprintf (stderr, _("Usage: %s [-bfhixV] [-o superblock=<num>] "
-		 "[-o blocksize=<num>] device\n"), program_name);
+	fprintf (stderr, _("Usage: %s [-bfhixV] [-ob superblock] "
+		 "[-oB blocksize] device\n"), program_name);
 	exit (1);
 }
 
@@ -126,24 +126,10 @@ static void print_bg_opts(ext2_filsys fs, dgrp_t i)
 	fputc('\n', stdout);
 }
 
-static void print_bg_rel_offset(ext2_filsys fs, blk64_t block, int itable,
-				blk64_t first_block, blk64_t last_block)
-{
-	if ((block >= first_block) && (block <= last_block)) {
-		if (itable && block == first_block)
-			return;
-		printf(" (+%u)", (unsigned)(block - first_block));
-	} else if (fs->super->s_feature_incompat &
-		   EXT4_FEATURE_INCOMPAT_FLEX_BG) {
-		dgrp_t flex_grp = ext2fs_group_of_blk(fs, block);
-		printf(" (bg #%u + %u)", flex_grp,
-		       (unsigned)(block-ext2fs_group_first_block(fs,flex_grp)));
-	}
-}
-
 static void list_desc (ext2_filsys fs)
 {
 	unsigned long i;
+	long diff;
 	blk_t	first_block, last_block;
 	blk_t	super_blk, old_desc_blk, new_desc_blk;
 	char *block_bitmap=NULL, *inode_bitmap=NULL;
@@ -213,21 +199,21 @@ static void list_desc (ext2_filsys fs)
 			fputc('\n', stdout);
 		fputs(_("  Block bitmap at "), stdout);
 		print_number(fs->group_desc[i].bg_block_bitmap);
-		print_bg_rel_offset(fs, fs->group_desc[i].bg_block_bitmap, 0,
-				    first_block, last_block);
-
+		diff = fs->group_desc[i].bg_block_bitmap - first_block;
+		if (diff >= 0)
+			printf(" (+%ld)", diff);
 		fputs(_(", Inode bitmap at "), stdout);
 		print_number(fs->group_desc[i].bg_inode_bitmap);
-		print_bg_rel_offset(fs, fs->group_desc[i].bg_inode_bitmap, 0,
-				    first_block, last_block);
-
+		diff = fs->group_desc[i].bg_inode_bitmap - first_block;
+		if (diff >= 0)
+			printf(" (+%ld)", diff);
 		fputs(_("\n  Inode table at "), stdout);
 		print_range(fs->group_desc[i].bg_inode_table,
 			    fs->group_desc[i].bg_inode_table +
 			    inode_blocks_per_group - 1);
-		print_bg_rel_offset(fs, fs->group_desc[i].bg_inode_table, 1,
-				    first_block, last_block);
-
+		diff = fs->group_desc[i].bg_inode_table - first_block;
+		if (diff > 0)
+			printf(" (+%ld)", diff);
 		printf (_("\n  %u free blocks, %u free inodes, "
 			  "%u directories%s"),
 			fs->group_desc[i].bg_free_blocks_count,
@@ -257,10 +243,6 @@ static void list_desc (ext2_filsys fs)
 			ino_itr += fs->super->s_inodes_per_group;
 		}
 	}
-	if (block_bitmap)
-		free(block_bitmap);
-	if (inode_bitmap)
-		free(inode_bitmap);
 }
 
 static void list_bad_blocks(ext2_filsys fs, int dump)
@@ -295,19 +277,14 @@ static void list_bad_blocks(ext2_filsys fs, int dump)
 	ext2fs_badblocks_list_iterate_end(bb_iter);
 	if (!dump)
 		fputc('\n', stdout);
-	ext2fs_badblocks_list_free(bb_list);
 }
 
 static void print_inline_journal_information(ext2_filsys fs)
 {
-	journal_superblock_t	*jsb;
 	struct ext2_inode	inode;
-	ext2_file_t		journal_file;
 	errcode_t		retval;
 	ino_t			ino = fs->super->s_journal_inum;
-	char			buf[1024];
-	__u32			*mask_ptr, mask, m;
-	int			i, j, size, printed = 0;
+	int			size;
 
 	retval = ext2fs_read_inode(fs, ino,  &inode);
 	if (retval) {
@@ -315,38 +292,6 @@ static void print_inline_journal_information(ext2_filsys fs)
 			_("while reading journal inode"));
 		exit(1);
 	}
-	retval = ext2fs_file_open2(fs, ino, &inode, 0, &journal_file);
-	if (retval) {
-		com_err(program_name, retval,
-			_("while opening journal inode"));
-		exit(1);
-	}
-	retval = ext2fs_file_read(journal_file, buf, sizeof(buf), 0);
-	if (retval) {
-		com_err(program_name, retval,
-			_("while reading journal super block"));
-		exit(1);
-	}
-	ext2fs_file_close(journal_file);
-	jsb = (journal_superblock_t *) buf;
-	if (be32_to_cpu(jsb->s_header.h_magic) != JFS_MAGIC_NUMBER) {
-		fprintf(stderr,
-			"Journal superblock magic number invalid!\n");
-		exit(1);
-	}
-	printf(_("Journal features:        "));
-	for (i=0, mask_ptr=&jsb->s_feature_compat; i <3; i++,mask_ptr++) {
-		mask = be32_to_cpu(*mask_ptr);
-		for (j=0,m=1; j < 32; j++, m<<=1) {
-			if (mask & m) {
-				printf(" %s", e2p_jrnl_feature2string(i, m));
-				printed++;
-			}
-		}
-	}
-	if (printed == 0)
-		printf(" (none)");
-	printf("\n");
 	fputs(_("Journal size:             "), stdout);
 	if ((fs->super->s_feature_ro_compat &
 	     EXT4_FEATURE_RO_COMPAT_HUGE_FILE) &&
@@ -358,12 +303,6 @@ static void print_inline_journal_information(ext2_filsys fs)
 		printf("%uk\n", size);
 	else
 		printf("%uM\n", size >> 10);
-	printf(_("Journal length:           %u\n"
-		 "Journal sequence:         0x%08x\n"
-		 "Journal start:            %u\n"),
-	       (unsigned int)ntohl(jsb->s_maxlen),
-	       (unsigned int)ntohl(jsb->s_sequence),
-	       (unsigned int)ntohl(jsb->s_start));
 }
 
 static void print_journal_information(ext2_filsys fs)
