@@ -25,7 +25,7 @@
  * 94/03/06	- Added the checks interval from Uwe Ohse (uwe@tirka.gun.de)
  */
 
-#define _XOPEN_SOURCE 500 /* for inclusion of strptime() */
+#define _XOPEN_SOURCE 600 /* for inclusion of strptime() */
 #define _BSD_SOURCE /* for inclusion of strcasecmp() */
 #include <fcntl.h>
 #include <grp.h>
@@ -37,7 +37,9 @@ extern int optind;
 #endif
 #include <pwd.h>
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
@@ -57,10 +59,10 @@ extern int optind;
 #include "../version.h"
 #include "nls-enable.h"
 
-const char * program_name = "tune2fs";
-char * device_name;
-char * new_label, *new_last_mounted, *new_UUID;
-char * io_options;
+const char *program_name = "tune2fs";
+char *device_name;
+char *new_label, *new_last_mounted, *new_UUID;
+char *io_options;
 static int c_flag, C_flag, e_flag, f_flag, g_flag, i_flag, l_flag, L_flag;
 static int m_flag, M_flag, r_flag, s_flag = -1, u_flag, U_flag, T_flag;
 static int I_flag;
@@ -108,7 +110,7 @@ static void usage(void)
 		  "\t[-M last_mounted_dir] [-O [^]feature[,...]]\n"
 		  "\t[-E extended-option[,...]] [-T last_check_time] "
 		  "[-U UUID]\n\t[ -I new_inode_size ] device\n"), program_name);
-	exit (1);
+	exit(1);
 }
 
 static __u32 ok_features[3] = {
@@ -172,11 +174,12 @@ static void remove_journal_device(ext2_filsys fs)
 	}
 
 #ifdef CONFIG_TESTIO_DEBUG
-	io_ptr = test_io_manager;
-	test_io_backing_manager = unix_io_manager;
-#else
-	io_ptr = unix_io_manager;
+	if (getenv("TEST_IO_FLAGS") || getenv("TEST_IO_BLOCK")) {
+		io_ptr = test_io_manager;
+		test_io_backing_manager = unix_io_manager;
+	} else
 #endif
+		io_ptr = unix_io_manager;
 	retval = ext2fs_open(journal_path, EXT2_FLAG_RW|
 			     EXT2_FLAG_JOURNAL_DEV_OK, 0,
 			     fs->blocksize, io_ptr, &jfs);
@@ -207,19 +210,19 @@ static void remove_journal_device(ext2_filsys fs)
 
 	/* Find the filesystem UUID */
 	nr_users = ntohl(jsb->s_nr_users);
-	for (i=0; i < nr_users; i++) {
+	for (i = 0; i < nr_users; i++) {
 		if (memcmp(fs->super->s_uuid,
 			   &jsb->s_users[i*16], 16) == 0)
 			break;
 	}
 	if (i >= nr_users) {
-		fputs(_("Filesystem's UUID not found on journal device.\n"), 
+		fputs(_("Filesystem's UUID not found on journal device.\n"),
 		      stderr);
 		commit_remove_journal = 1;
 		goto no_valid_journal;
 	}
 	nr_users--;
-	for (i=0; i < nr_users; i++)
+	for (i = 0; i < nr_users; i++)
 		memcpy(&jsb->s_users[i*16], &jsb->s_users[(i+1)*16], 16);
 	jsb->s_nr_users = htonl(nr_users);
 
@@ -246,14 +249,14 @@ no_valid_journal:
 
 /* Helper function for remove_journal_inode */
 static int release_blocks_proc(ext2_filsys fs, blk_t *blocknr,
-			       int blockcnt EXT2FS_ATTR((unused)), 
+			       int blockcnt EXT2FS_ATTR((unused)),
 			       void *private EXT2FS_ATTR((unused)))
 {
 	blk_t	block;
 	int	group;
 
 	block = *blocknr;
-	ext2fs_unmark_block_bitmap(fs->block_map,block);
+	ext2fs_unmark_block_bitmap(fs->block_map, block);
 	group = ext2fs_group_of_blk(fs, block);
 	fs->group_desc[group].bg_free_blocks_count++;
 	ext2fs_group_desc_csum_set(fs, group);
@@ -269,7 +272,7 @@ static void remove_journal_inode(ext2_filsys fs)
 	struct ext2_inode	inode;
 	errcode_t		retval;
 	ino_t			ino = fs->super->s_journal_inum;
-	
+
 	retval = ext2fs_read_inode(fs, ino,  &inode);
 	if (retval) {
 		com_err(program_name, retval,
@@ -311,7 +314,7 @@ static void remove_journal_inode(ext2_filsys fs)
  */
 static void update_mntopts(ext2_filsys fs, char *mntopts)
 {
-	struct ext2_super_block *sb= fs->super;
+	struct ext2_super_block *sb = fs->super;
 
 	if (e2p_edit_mntopts(mntopts, &sb->s_default_mount_opts, ~0)) {
 		fprintf(stderr, _("Invalid mount option set: %s\n"),
@@ -326,7 +329,7 @@ static void update_mntopts(ext2_filsys fs, char *mntopts)
  */
 static void update_feature_set(ext2_filsys fs, char *features)
 {
-	struct ext2_super_block *sb= fs->super;
+	struct ext2_super_block *sb = fs->super;
 	__u32		old_features[3];
 	int		type_err;
 	unsigned int	mask_err;
@@ -389,9 +392,8 @@ static void update_feature_set(ext2_filsys fs, char *features)
 	if (FEATURE_ON(E2P_FEATURE_COMPAT, EXT3_FEATURE_COMPAT_HAS_JOURNAL)) {
 		/*
 		 * If adding a journal flag, let the create journal
-		 * code below handle creating setting the flag and
-		 * creating the journal.  We supply a default size if
-		 * necessary.
+		 * code below handle setting the flag and creating the
+		 * journal.  We supply a default size if necessary.
 		 */
 		if (!journal_size)
 			journal_size = -1;
@@ -400,7 +402,7 @@ static void update_feature_set(ext2_filsys fs, char *features)
 
 	if (FEATURE_ON(E2P_FEATURE_COMPAT, EXT2_FEATURE_COMPAT_DIR_INDEX)) {
 		if (!sb->s_def_hash_version)
-			sb->s_def_hash_version = EXT2_HASH_TEA;
+			sb->s_def_hash_version = EXT2_HASH_HALF_MD4;
 		if (uuid_is_null((unsigned char *) sb->s_hash_seed))
 			uuid_generate((unsigned char *) sb->s_hash_seed);
 	}
@@ -474,11 +476,12 @@ static void add_journal(ext2_filsys fs)
 		check_plausibility(journal_device);
 		check_mount(journal_device, 0, _("journal"));
 #ifdef CONFIG_TESTIO_DEBUG
-		io_ptr = test_io_manager;
-		test_io_backing_manager = unix_io_manager;
-#else
-		io_ptr = unix_io_manager;
+		if (getenv("TEST_IO_FLAGS") || getenv("TEST_IO_BLOCK")) {
+			io_ptr = test_io_manager;
+			test_io_backing_manager = unix_io_manager;
+		} else
 #endif
+			io_ptr = unix_io_manager;
 		retval = ext2fs_open(journal_device, EXT2_FLAG_RW|
 				     EXT2_FLAG_JOURNAL_DEV_OK, 0,
 				     fs->blocksize, io_ptr, &jfs);
@@ -495,9 +498,9 @@ static void add_journal(ext2_filsys fs)
 		retval = ext2fs_add_journal_device(fs, jfs);
 		ext2fs_close(jfs);
 		if (retval) {
-			com_err (program_name, retval,
-				 _("while adding filesystem to journal on %s"),
-				 journal_device);
+			com_err(program_name, retval,
+				_("while adding filesystem to journal on %s"),
+				journal_device);
 			goto err;
 		}
 		fputs(_("done\n"), stdout);
@@ -543,7 +546,7 @@ static void parse_e2label_options(int argc, char ** argv)
 		*io_options++ = 0;
 	device_name = blkid_get_devname(NULL, argv[1], NULL);
 	if (!device_name) {
-		com_err("e2label", 0, _("Unable to resolve '%s'"), 
+		com_err("e2label", 0, _("Unable to resolve '%s'"),
 			argv[1]);
 		exit(1);
 	}
@@ -552,7 +555,7 @@ static void parse_e2label_options(int argc, char ** argv)
 		open_flag |= EXT2_FLAG_RW;
 		L_flag = 1;
 		new_label = argv[2];
-	} else 
+	} else
 		print_label++;
 }
 
@@ -589,192 +592,191 @@ static time_t parse_time(char *str)
 static void parse_tune2fs_options(int argc, char **argv)
 {
 	int c;
-	char * tmp;
-	struct group * gr;
-	struct passwd * pw;
+	char *tmp;
+	struct group *gr;
+	struct passwd *pw;
 
 	open_flag = 0;
 
 	printf("tune2fs %s (%s)\n", E2FSPROGS_VERSION, E2FSPROGS_DATE);
 	while ((c = getopt(argc, argv, "c:e:fg:i:jlm:o:r:s:u:C:E:I:J:L:M:O:T:U:")) != EOF)
-		switch (c)
-		{
-			case 'c':
-				max_mount_count = strtol (optarg, &tmp, 0);
-				if (*tmp || max_mount_count > 16000) {
-					com_err (program_name, 0,
-						 _("bad mounts count - %s"),
-						 optarg);
-					usage();
-				}
-				if (max_mount_count == 0)
-					max_mount_count = -1;
-				c_flag = 1;
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'C':
-				mount_count = strtoul (optarg, &tmp, 0);
-				if (*tmp || mount_count > 16000) {
-					com_err (program_name, 0,
-						 _("bad mounts count - %s"),
-						 optarg);
-					usage();
-				}
-				C_flag = 1;
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'e':
-				if (strcmp (optarg, "continue") == 0)
-					errors = EXT2_ERRORS_CONTINUE;
-				else if (strcmp (optarg, "remount-ro") == 0)
-					errors = EXT2_ERRORS_RO;
-				else if (strcmp (optarg, "panic") == 0)
-					errors = EXT2_ERRORS_PANIC;
+		switch (c) {
+		case 'c':
+			max_mount_count = strtol(optarg, &tmp, 0);
+			if (*tmp || max_mount_count > 16000) {
+				com_err(program_name, 0,
+					_("bad mounts count - %s"),
+					optarg);
+				usage();
+			}
+			if (max_mount_count == 0)
+				max_mount_count = -1;
+			c_flag = 1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'C':
+			mount_count = strtoul(optarg, &tmp, 0);
+			if (*tmp || mount_count > 16000) {
+				com_err(program_name, 0,
+					_("bad mounts count - %s"),
+					optarg);
+				usage();
+			}
+			C_flag = 1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'e':
+			if (strcmp(optarg, "continue") == 0)
+				errors = EXT2_ERRORS_CONTINUE;
+			else if (strcmp(optarg, "remount-ro") == 0)
+				errors = EXT2_ERRORS_RO;
+			else if (strcmp(optarg, "panic") == 0)
+				errors = EXT2_ERRORS_PANIC;
+			else {
+				com_err(program_name, 0,
+					_("bad error behavior - %s"),
+					optarg);
+				usage();
+			}
+			e_flag = 1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'E':
+			extended_cmd = optarg;
+			open_flag |= EXT2_FLAG_RW;
+			break;
+		case 'f': /* Force */
+			f_flag = 1;
+			break;
+		case 'g':
+			resgid = strtoul(optarg, &tmp, 0);
+			if (*tmp) {
+				gr = getgrnam(optarg);
+				if (gr == NULL)
+					tmp = optarg;
 				else {
-					com_err (program_name, 0,
-						 _("bad error behavior - %s"),
-						 optarg);
-					usage();
+					resgid = gr->gr_gid;
+					*tmp = 0;
 				}
-				e_flag = 1;
-				open_flag = EXT2_FLAG_RW;
+			}
+			if (*tmp) {
+				com_err(program_name, 0,
+					_("bad gid/group name - %s"),
+					optarg);
+				usage();
+			}
+			g_flag = 1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'i':
+			interval = strtoul(optarg, &tmp, 0);
+			switch (*tmp) {
+			case 's':
+				tmp++;
 				break;
-			case 'E':
-				extended_cmd = optarg;
-				open_flag |= EXT2_FLAG_RW;
-				break;
-			case 'f': /* Force */
-				f_flag = 1;
-				break;
-			case 'g':
-				resgid = strtoul (optarg, &tmp, 0);
-				if (*tmp) {
-					gr = getgrnam (optarg);
-					if (gr == NULL)
-						tmp = optarg;
-					else {
-						resgid = gr->gr_gid;
-						*tmp =0;
-					}
-				}
-				if (*tmp) {
-					com_err (program_name, 0,
-						 _("bad gid/group name - %s"),
-						 optarg);
-					usage();
-				}
-				g_flag = 1;
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'i':
-				interval = strtoul (optarg, &tmp, 0);
-				switch (*tmp) {
-				case 's':
+			case '\0':
+			case 'd':
+			case 'D': /* days */
+				interval *= 86400;
+				if (*tmp != '\0')
 					tmp++;
-					break;
-				case '\0':
-				case 'd':
-				case 'D': /* days */
-					interval *= 86400;
-					if (*tmp != '\0')
-						tmp++;
-					break;
-				case 'm':
-				case 'M': /* months! */
-					interval *= 86400 * 30;
-					tmp++;
-					break;
-				case 'w':
-				case 'W': /* weeks */
-					interval *= 86400 * 7;
-					tmp++;
-					break;
-				}
-				if (*tmp) {
-					com_err (program_name, 0,
-						_("bad interval - %s"), optarg);
-					usage();
-				}
-				i_flag = 1;
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'j':
-				if (!journal_size)
-					journal_size = -1;
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'J':
-				parse_journal_opts(optarg);
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'l':
-				l_flag = 1;
-				break;
-			case 'L':
-				new_label = optarg;
-				L_flag = 1;
-				open_flag |= EXT2_FLAG_RW |
-					EXT2_FLAG_JOURNAL_DEV_OK;
 				break;
 			case 'm':
-				reserved_ratio = strtod(optarg, &tmp);
-				if (*tmp || reserved_ratio > 50) {
-					com_err (program_name, 0,
-						 _("bad reserved block ratio - %s"),
-						 optarg);
-					usage();
-				}
-				m_flag = 1;
-				open_flag = EXT2_FLAG_RW;
+			case 'M': /* months! */
+				interval *= 86400 * 30;
+				tmp++;
 				break;
-			case 'M':
-				new_last_mounted = optarg;
-				M_flag = 1;
-				open_flag = EXT2_FLAG_RW;
+			case 'w':
+			case 'W': /* weeks */
+				interval *= 86400 * 7;
+				tmp++;
 				break;
-			case 'o':
-				if (mntopts_cmd) {
-					com_err (program_name, 0,
-					 _("-o may only be specified once"));
-					usage();
-				}
-				mntopts_cmd = optarg;
-				open_flag = EXT2_FLAG_RW;
-				break;
-				
-			case 'O':
-				if (features_cmd) {
-					com_err (program_name, 0,
-					 _("-O may only be specified once"));
-					usage();
-				}
-				features_cmd = optarg;
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'r':
-				reserved_blocks = strtoul (optarg, &tmp, 0);
+			}
+			if (*tmp) {
+				com_err(program_name, 0,
+					_("bad interval - %s"), optarg);
+				usage();
+			}
+			i_flag = 1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'j':
+			if (!journal_size)
+				journal_size = -1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'J':
+			parse_journal_opts(optarg);
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'l':
+			l_flag = 1;
+			break;
+		case 'L':
+			new_label = optarg;
+			L_flag = 1;
+			open_flag |= EXT2_FLAG_RW |
+				EXT2_FLAG_JOURNAL_DEV_OK;
+			break;
+		case 'm':
+			reserved_ratio = strtod(optarg, &tmp);
+			if (*tmp || reserved_ratio > 50) {
+				com_err(program_name, 0,
+					_("bad reserved block ratio - %s"),
+					optarg);
+				usage();
+			}
+			m_flag = 1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'M':
+			new_last_mounted = optarg;
+			M_flag = 1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'o':
+			if (mntopts_cmd) {
+				com_err(program_name, 0,
+					_("-o may only be specified once"));
+				usage();
+			}
+			mntopts_cmd = optarg;
+			open_flag = EXT2_FLAG_RW;
+			break;
+			
+		case 'O':
+			if (features_cmd) {
+				com_err(program_name, 0,
+					_("-O may only be specified once"));
+				usage();
+			}
+			features_cmd = optarg;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'r':
+			reserved_blocks = strtoul(optarg, &tmp, 0);
+			if (*tmp) {
+				com_err(program_name, 0,
+					_("bad reserved blocks count - %s"),
+					optarg);
+				usage();
+			}
+			r_flag = 1;
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 's': /* Deprecated */
+			s_flag = atoi(optarg);
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'T':
+			T_flag = 1;
+			last_check_time = parse_time(optarg);
+			open_flag = EXT2_FLAG_RW;
+			break;
+		case 'u':
+				resuid = strtoul(optarg, &tmp, 0);
 				if (*tmp) {
-					com_err (program_name, 0,
-						 _("bad reserved blocks count - %s"),
-						 optarg);
-					usage();
-				}
-				r_flag = 1;
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 's': /* Deprecated */
-				s_flag = atoi(optarg);
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'T':
-				T_flag = 1;
-				last_check_time = parse_time(optarg);
-				open_flag = EXT2_FLAG_RW;
-				break;
-			case 'u':
-				resuid = strtoul (optarg, &tmp, 0);
-				if (*tmp) {
-					pw = getpwnam (optarg);
+					pw = getpwnam(optarg);
 					if (pw == NULL)
 						tmp = optarg;
 					else {
@@ -783,41 +785,41 @@ static void parse_tune2fs_options(int argc, char **argv)
 					}
 				}
 				if (*tmp) {
-					com_err (program_name, 0,
-						 _("bad uid/user name - %s"),
-						 optarg);
+					com_err(program_name, 0,
+						_("bad uid/user name - %s"),
+						optarg);
 					usage();
 				}
 				u_flag = 1;
 				open_flag = EXT2_FLAG_RW;
 				break;
-			case 'U':
-				new_UUID = optarg;
-				U_flag = 1;
-				open_flag = EXT2_FLAG_RW |
-					EXT2_FLAG_JOURNAL_DEV_OK;
-				break;
-			case 'I':
-				new_inode_size = strtoul (optarg, &tmp, 0);
-				if (*tmp) {
-					com_err (program_name, 0,
-						_("bad inode size - %s"),
-							optarg);
-					usage();
-				}
-				if (!((new_inode_size &
-						(new_inode_size - 1)) == 0)) {
-					com_err (program_name, 0,
-						_("Inode size must be a "
-						"power of two- %s"),
-							optarg);
-					usage();
-				}
-				open_flag = EXT2_FLAG_RW;
-				I_flag = 1;
-				break;
-			default:
+		case 'U':
+			new_UUID = optarg;
+			U_flag = 1;
+			open_flag = EXT2_FLAG_RW |
+				EXT2_FLAG_JOURNAL_DEV_OK;
+			break;
+		case 'I':
+			new_inode_size = strtoul(optarg, &tmp, 0);
+			if (*tmp) {
+				com_err(program_name, 0,
+					_("bad inode size - %s"),
+					optarg);
 				usage();
+			}
+			if (!((new_inode_size &
+			       (new_inode_size - 1)) == 0)) {
+				com_err(program_name, 0,
+					_("Inode size must be a "
+					  "power of two- %s"),
+					optarg);
+				usage();
+			}
+			open_flag = EXT2_FLAG_RW;
+			I_flag = 1;
+			break;
+		default:
+			usage();
 		}
 	if (optind < argc - 1 || optind == argc)
 		usage();
@@ -828,7 +830,7 @@ static void parse_tune2fs_options(int argc, char **argv)
 		*io_options++ = 0;
 	device_name = blkid_get_devname(NULL, argv[optind], NULL);
 	if (!device_name) {
-		com_err("tune2fs", 0, _("Unable to resolve '%s'"), 
+		com_err("tune2fs", 0, _("Unable to resolve '%s'"),
 			argv[optind]);
 		exit(1);
 	}
@@ -845,7 +847,7 @@ void do_findfs(int argc, char **argv)
 	}
 	dev = blkid_get_devname(NULL, argv[1], NULL);
 	if (!dev) {
-		com_err("findfs", 0, _("Unable to resolve '%s'"), 
+		com_err("findfs", 0, _("Unable to resolve '%s'"),
 			argv[1]);
 		exit(1);
 	}
@@ -856,7 +858,7 @@ void do_findfs(int argc, char **argv)
 static void parse_extended_opts(ext2_filsys fs, const char *opts)
 {
 	char	*buf, *token, *next, *p, *arg;
-	int	len;
+	int	len, hash_alg;
 	int	r_usage = 0;
 
 	len = strlen(opts);
@@ -916,7 +918,26 @@ static void parse_extended_opts(ext2_filsys fs, const char *opts)
 				continue;
 			}
 			stripe_width_set = 1;
-		} else 
+		} else if (strcmp(token, "hash_alg") == 0 ||
+			   strcmp(token, "hash-alg") == 0) {
+			if (!arg) {
+				r_usage++;
+				continue;
+			}
+			hash_alg = e2p_string2hash(arg);
+			if (hash_alg < 0) {
+				fprintf(stderr,
+					_("Invalid hash algorithm: %s\n"),
+					arg);
+				r_usage++;
+				continue;
+			}
+			fs->super->s_def_hash_version = hash_alg;
+			printf(_("Setting default hash algorithm "
+				 "to %s (%d)\n"),
+			       arg, hash_alg);
+			ext2fs_mark_super_dirty(fs);
+		} else
 			r_usage++;
 	}
 	if (r_usage) {
@@ -926,23 +947,37 @@ static void parse_extended_opts(ext2_filsys fs, const char *opts)
 			"\tis set off by an equals ('=') sign.\n\n"
 			"Valid extended options are:\n"
 			"\tstride=<RAID per-disk chunk size in blocks>\n"
-			"\tstripe-width=<RAID stride*data disks in blocks>\n"
+			"\tstripe_width=<RAID stride*data disks in blocks>\n"
+			"\thash_alg=<hash algorithm>\n"
 			"\ttest_fs\n"
 			"\t^test_fs\n"));
 		free(buf);
 		exit(1);
 	}
 	free(buf);
-}	
+}
 
-static int get_move_bitmap(ext2_filsys fs, int new_ino_blks_per_grp,
-					ext2fs_block_bitmap bmap)
+/*
+ * Fill in two bitmaps that we need to to control the inode resizing
+ * process.  The first is the set of blocks that must be moved, and
+ * the second is the set of blocks which are allocation bitmap blocks
+ * and must be treated specially.
+ */
+static int get_move_bitmaps(ext2_filsys fs, int new_ino_blks_per_grp,
+			    ext2fs_block_bitmap bmap,
+			    ext2fs_block_bitmap metadata_bmap)
 {
 	dgrp_t i;
 	blk_t j, needed_blocks = 0;
+	blk_t i_bmap, b_bmap;
 	blk_t start_blk, end_blk;
+	int num, k;
 
 	for (i = 0; i < fs->group_desc_count; i++) {
+		b_bmap = fs->group_desc[i].bg_block_bitmap;
+		ext2fs_mark_block_bitmap(metadata_bmap, b_bmap);
+		i_bmap = fs->group_desc[i].bg_inode_bitmap;
+		ext2fs_mark_block_bitmap(metadata_bmap, i_bmap);
 
 		start_blk = fs->group_desc[i].bg_inode_table +
 					fs->inode_blocks_per_group;
@@ -950,8 +985,8 @@ static int get_move_bitmap(ext2_filsys fs, int new_ino_blks_per_grp,
 		end_blk = fs->group_desc[i].bg_inode_table +
 					new_ino_blks_per_grp;
 
+		num=0;
 		for (j = start_blk; j < end_blk; j++) {
-
 			if (ext2fs_test_block_bitmap(fs->block_map, j)) {
 				/* FIXME!!
 				 * What happens if the block is marked
@@ -966,35 +1001,48 @@ static int get_move_bitmap(ext2_filsys fs, int new_ino_blks_per_grp,
 				 */
 				ext2fs_mark_block_bitmap(fs->block_map, j);
 			}
+			if ((j == i_bmap) || (j == b_bmap))
+				num++;
 		}
+		if (num <= fs->group_desc[i].bg_free_blocks_count) 
+			continue;
 	}
 
-	if (needed_blocks > fs->super->s_free_blocks_count ) {
+	if (needed_blocks > fs->super->s_free_blocks_count)
 		return ENOSPC;
-	}
 
 	return 0;
 }
 
-static int move_block(ext2_filsys fs, ext2fs_block_bitmap bmap)
+static int move_block(ext2_filsys fs, ext2fs_block_bitmap bmap,
+		      ext2fs_block_bitmap metadata_bmap)
 {
 	char *buf;
 	errcode_t retval;
-	blk_t blk, new_blk;
+	blk_t blk, new_blk, goal;
 	struct blk_move *bmv;
-
+	dgrp_t		group;
 
 	retval = ext2fs_get_mem(fs->blocksize, &buf);
 	if (retval)
 		return retval;
 
-	for (blk = fs->super->s_first_data_block;
-			blk < fs->super->s_blocks_count; blk++) {
-
+	for (new_blk = blk = fs->super->s_first_data_block;
+	     blk < fs->super->s_blocks_count; blk++) {
 		if (!ext2fs_test_block_bitmap(bmap, blk))
 			continue;
 
-		retval = ext2fs_new_block(fs, blk, NULL, &new_blk);
+		/*
+		 * If the block is a bitmap block, find a new block in
+		 * the same block group.
+		 */
+		if (ext2fs_test_block_bitmap(metadata_bmap, blk)) {
+			group = ext2fs_group_of_blk(fs, blk);
+			goal = ext2fs_group_first_block(fs, group);
+		} else
+			goal = new_blk;
+
+		retval = ext2fs_new_block(fs, goal, NULL, &new_blk);
 		if (retval)
 			goto err_out;
 
@@ -1025,13 +1073,12 @@ err_out:
 	return retval;
 }
 
-static blk_t transalate_block(blk_t blk)
+static blk_t translate_block(blk_t blk)
 {
 	struct list_head *entry;
 	struct blk_move *bmv;
 
 	list_for_each(entry, &blk_move_list) {
-
 		bmv = list_entry(entry, struct blk_move, list);
 		if (bmv->old_loc == blk)
 			return bmv->new_loc;
@@ -1045,13 +1092,15 @@ static int process_block(ext2_filsys fs EXT2FS_ATTR((unused)),
 			 e2_blkcnt_t blockcnt EXT2FS_ATTR((unused)),
 			 blk_t ref_block EXT2FS_ATTR((unused)),
 			 int ref_offset EXT2FS_ATTR((unused)),
-			 void *priv_data EXT2FS_ATTR((unused)))
+			 void *priv_data)
 {
 	int ret = 0;
 	blk_t new_blk;
+	ext2fs_block_bitmap bmap = (ext2fs_block_bitmap) priv_data;
 
-
-	new_blk = transalate_block(*block_nr);
+	if (!ext2fs_test_block_bitmap(bmap, *block_nr))
+		return 0;
+	new_blk = translate_block(*block_nr);
 	if (new_blk) {
 		*block_nr = new_blk;
 		/*
@@ -1063,7 +1112,7 @@ static int process_block(ext2_filsys fs EXT2FS_ATTR((unused)),
 	return ret;
 }
 
-static int inode_scan_and_fix(ext2_filsys fs)
+static int inode_scan_and_fix(ext2_filsys fs, ext2fs_block_bitmap bmap)
 {
 	errcode_t retval = 0;
 	ext2_ino_t ino;
@@ -1081,7 +1130,6 @@ static int inode_scan_and_fix(ext2_filsys fs)
 		goto err_out;
 
 	while (1) {
-
 		retval = ext2fs_get_next_inode(scan, &ino, &inode);
 		if (retval)
 			goto err_out;
@@ -1099,9 +1147,9 @@ static int inode_scan_and_fix(ext2_filsys fs)
 		 * Do we need to fix this ??
 		 */
 
-		if (inode.i_file_acl) {
-
-			blk = transalate_block(inode.i_file_acl);
+		if (inode.i_file_acl &&
+		    ext2fs_test_block_bitmap(bmap, inode.i_file_acl)) {
+			blk = translate_block(inode.i_file_acl);
 			if (!blk)
 				continue;
 
@@ -1119,9 +1167,8 @@ static int inode_scan_and_fix(ext2_filsys fs)
 		if (!ext2fs_inode_has_valid_blocks(&inode))
 			continue;
 
-		retval = ext2fs_block_iterate2(fs, ino, 0,
-						block_buf, process_block,
-						0);
+		retval = ext2fs_block_iterate2(fs, ino, 0, block_buf,
+					       process_block, bmap);
 		if (retval)
 			goto err_out;
 
@@ -1131,9 +1178,39 @@ err_out:
 	ext2fs_free_mem(&block_buf);
 
 	return retval;
-
 }
 
+/*
+ * We need to scan for inode and block bitmaps that may need to be
+ * moved.  This can take place if the filesystem was formatted for
+ * RAID arrays using the mke2fs's extended option "stride".
+ */
+static int group_desc_scan_and_fix(ext2_filsys fs, ext2fs_block_bitmap bmap)
+{
+	dgrp_t i;
+	blk_t blk, new_blk;
+
+	for (i = 0; i < fs->group_desc_count; i++) {
+		blk = fs->group_desc[i].bg_block_bitmap;
+		if (ext2fs_test_block_bitmap(bmap, blk)) {
+			new_blk = translate_block(blk);
+			if (!new_blk)
+				continue;
+			fs->group_desc[i].bg_block_bitmap = new_blk;
+		}
+
+		blk = fs->group_desc[i].bg_inode_bitmap;
+		if (ext2fs_test_block_bitmap(bmap, blk)) {
+			new_blk = translate_block(blk);
+			if (!new_blk)
+				continue;
+			fs->group_desc[i].bg_inode_bitmap = new_blk;
+		}
+	}
+	return 0;
+}
+
+
 static int expand_inode_table(ext2_filsys fs, unsigned long new_ino_size)
 {
 	dgrp_t i;
@@ -1168,7 +1245,6 @@ static int expand_inode_table(ext2_filsys fs, unsigned long new_ino_size)
 	tmp_new_itable = new_itable;
 
 	for (i = 0; i < fs->group_desc_count; i++) {
-
 		blk = fs->group_desc[i].bg_inode_table;
 		retval = io_channel_read_blk(fs->io, blk,
 				fs->inode_blocks_per_group, old_itable);
@@ -1176,7 +1252,6 @@ static int expand_inode_table(ext2_filsys fs, unsigned long new_ino_size)
 			goto err_out;
 
 		for (j = 0; j < EXT2_INODES_PER_GROUP(fs->super); j++) {
-
 			memcpy(new_itable, old_itable, old_ino_size);
 
 			memset(new_itable+old_ino_size, 0,
@@ -1277,25 +1352,18 @@ static void free_blk_move_list(void)
 	struct blk_move *bmv;
 
 	list_for_each_safe(entry, tmp, &blk_move_list) {
-
 		bmv = list_entry(entry, struct blk_move, list);
 		list_del(entry);
 		ext2fs_free_mem(&bmv);
 	}
-
-	return ;
+	return;
 }
 
 static int resize_inode(ext2_filsys fs, unsigned long new_size)
 {
 	errcode_t retval;
 	int new_ino_blks_per_grp;
-	ext2fs_block_bitmap bmap;
-
-	if (new_size <= EXT2_INODE_SIZE(fs->super)) {
-		fprintf(stderr, _("New inode size too small\n"));
-		return EXT2_ET_INVALID_ARGUMENT;
-	}
+	ext2fs_block_bitmap bmap, metadata_bmap;
 
 	ext2fs_read_inode_bitmap(fs);
 	ext2fs_read_block_bitmap(fs);
@@ -1318,15 +1386,25 @@ static int resize_inode(ext2_filsys fs, unsigned long new_size)
 	if (retval)
 		return retval;
 
-	retval = get_move_bitmap(fs, new_ino_blks_per_grp, bmap);
+	retval = ext2fs_allocate_block_bitmap(fs, _("blocks to be moved"),
+						&metadata_bmap);
+	if (retval)
+		return retval;
+
+	retval = get_move_bitmaps(fs, new_ino_blks_per_grp, bmap,
+				  metadata_bmap);
 	if (retval)
 		goto err_out;
 
-	retval = move_block(fs, bmap);
+	retval = move_block(fs, bmap, metadata_bmap);
 	if (retval)
 		goto err_out;
 
-	retval = inode_scan_and_fix(fs);
+	retval = inode_scan_and_fix(fs, bmap);
+	if (retval)
+		goto err_out;
+
+	retval = group_desc_scan_and_fix(fs, bmap);
 	if (retval)
 		goto err_out;
 
@@ -1369,7 +1447,7 @@ static int tune2fs_setup_tdb(const char *name, io_manager *io_ptr)
 
 	tdb_dir = getenv("E2FSPROGS_UNDO_DIR");
 	if (!tdb_dir)
-		tdb_dir="/var/lib/e2fsprogs";
+		tdb_dir = "/var/lib/e2fsprogs";
 
 	if (!strcmp(tdb_dir, "none") || (tdb_dir[0] == 0) ||
 	    access(tdb_dir, W_OK))
@@ -1390,19 +1468,19 @@ static int tune2fs_setup_tdb(const char *name, io_manager *io_ptr)
 	set_undo_io_backing_manager(*io_ptr);
 	*io_ptr = undo_io_manager;
 	set_undo_io_backup_file(tdb_file);
-	printf(_("To undo the tune2fs operations please run "
-		 "the command\n    undoe2fs %s %s\n\n"),
+	printf(_("To undo the tune2fs operation please run "
+		 "the command\n    e2undo %s %s\n\n"),
 		 tdb_file, name);
 	free(tmp_name);
 	return retval;
 }
 
-int main (int argc, char ** argv)
+int main(int argc, char **argv)
 {
 	errcode_t retval;
 	ext2_filsys fs;
 	struct ext2_super_block *sb;
-	io_manager io_ptr;
+	io_manager io_ptr, io_ptr_orig = NULL;
 
 #ifdef ENABLE_NLS
 	setlocale(LC_MESSAGES, "");
@@ -1420,33 +1498,58 @@ int main (int argc, char ** argv)
 		parse_e2label_options(argc, argv);
 	else
 		parse_tune2fs_options(argc, argv);
-	
+
 #ifdef CONFIG_TESTIO_DEBUG
-	io_ptr = test_io_manager;
-	test_io_backing_manager = unix_io_manager;
-#else
-	io_ptr = unix_io_manager;
+	if (getenv("TEST_IO_FLAGS") || getenv("TEST_IO_DEBUG")) {
+		io_ptr = test_io_manager;
+		test_io_backing_manager = unix_io_manager;
+	} else
 #endif
+		io_ptr = unix_io_manager;
 
-	if (I_flag) {
-		/*
-		 * If inode resize is requested use the
-		 * Undo I/O manager
-		 */
-		retval = tune2fs_setup_tdb(device_name, &io_ptr);
-		if (retval)
-			exit(1);
-	}
-
-	retval = ext2fs_open2(device_name, io_options, open_flag, 
+retry_open:
+	retval = ext2fs_open2(device_name, io_options, open_flag,
 			      0, 0, io_ptr, &fs);
-        if (retval) {
-		com_err (program_name, retval, _("while trying to open %s"),
-			 device_name);
+	if (retval) {
+			com_err(program_name, retval,
+				_("while trying to open %s"),
+			device_name);
 		fprintf(stderr,
 			_("Couldn't find valid filesystem superblock.\n"));
 		exit(1);
 	}
+
+	if (I_flag && !io_ptr_orig) {
+		/*
+		 * Check the inode size is right so we can issue an
+		 * error message and bail before setting up the tdb
+		 * file.
+		 */
+		if (new_inode_size == EXT2_INODE_SIZE(fs->super)) {
+			fprintf(stderr, _("The inode size is already %d\n"),
+				new_inode_size);
+			exit(1);
+		}
+		if (new_inode_size < EXT2_INODE_SIZE(fs->super)) {
+			fprintf(stderr, _("Shrinking the inode size is "
+					  "not supported\n"));
+			exit(1);
+		}
+
+		/*
+		 * If inode resize is requested use the
+		 * Undo I/O manager
+		 */
+		io_ptr_orig = io_ptr;
+		retval = tune2fs_setup_tdb(device_name, &io_ptr);
+		if (retval)
+			exit(1);
+		if (io_ptr != io_ptr_orig) {
+			ext2fs_close(fs);
+			goto retry_open;
+		}
+	}
+
 	sb = fs->super;
 	fs->flags &= ~EXT2_FLAG_MASTER_SB_ONLY;
 
@@ -1471,47 +1574,49 @@ int main (int argc, char ** argv)
 	if (c_flag) {
 		sb->s_max_mnt_count = max_mount_count;
 		ext2fs_mark_super_dirty(fs);
-		printf (_("Setting maximal mount count to %d\n"),
-			max_mount_count);
+		printf(_("Setting maximal mount count to %d\n"),
+		       max_mount_count);
 	}
 	if (C_flag) {
 		sb->s_mnt_count = mount_count;
 		ext2fs_mark_super_dirty(fs);
-		printf (_("Setting current mount count to %d\n"), mount_count);
+		printf(_("Setting current mount count to %d\n"), mount_count);
 	}
 	if (e_flag) {
 		sb->s_errors = errors;
 		ext2fs_mark_super_dirty(fs);
-		printf (_("Setting error behavior to %d\n"), errors);
+		printf(_("Setting error behavior to %d\n"), errors);
 	}
 	if (g_flag) {
 		sb->s_def_resgid = resgid;
 		ext2fs_mark_super_dirty(fs);
-		printf (_("Setting reserved blocks gid to %lu\n"), resgid);
+		printf(_("Setting reserved blocks gid to %lu\n"), resgid);
 	}
 	if (i_flag) {
 		sb->s_checkinterval = interval;
 		ext2fs_mark_super_dirty(fs);
-		printf (_("Setting interval between checks to %lu seconds\n"), interval);
+		printf(_("Setting interval between checks to %lu seconds\n"),
+		       interval);
 	}
 	if (m_flag) {
 		sb->s_r_blocks_count = (unsigned int) (reserved_ratio *
 					sb->s_blocks_count / 100.0);
 		ext2fs_mark_super_dirty(fs);
-		printf (_("Setting reserved blocks percentage to %g%% (%u blocks)\n"),
-			reserved_ratio, sb->s_r_blocks_count);
+		printf(_("Setting reserved blocks percentage to %g%% "
+			 "(%u blocks)\n"),
+		       reserved_ratio, sb->s_r_blocks_count);
 	}
 	if (r_flag) {
 		if (reserved_blocks >= sb->s_blocks_count/2) {
-			com_err (program_name, 0,
-				 _("reserved blocks count is too big (%lu)"),
-				 reserved_blocks);
-			exit (1);
+			com_err(program_name, 0,
+				_("reserved blocks count is too big (%lu)"),
+				reserved_blocks);
+			exit(1);
 		}
 		sb->s_r_blocks_count = reserved_blocks;
 		ext2fs_mark_super_dirty(fs);
-		printf (_("Setting reserved blocks count to %lu\n"),
-			reserved_blocks);
+		printf(_("Setting reserved blocks count to %lu\n"),
+		       reserved_blocks);
 	}
 	if (s_flag == 1) {
 		if (sb->s_feature_ro_compat &
@@ -1541,11 +1646,11 @@ int main (int argc, char ** argv)
 	if (u_flag) {
 		sb->s_def_resuid = resuid;
 		ext2fs_mark_super_dirty(fs);
-		printf (_("Setting reserved blocks uid to %lu\n"), resuid);
+		printf(_("Setting reserved blocks uid to %lu\n"), resuid);
 	}
 	if (L_flag) {
 		if (strlen(new_label) > sizeof(sb->s_volume_name))
-			fputs(_("Warning: label too long, truncating.\n"), 
+			fputs(_("Warning: label too long, truncating.\n"),
 			      stderr);
 		memset(sb->s_volume_name, 0, sizeof(sb->s_volume_name));
 		strncpy(sb->s_volume_name, new_label,
@@ -1566,8 +1671,24 @@ int main (int argc, char ** argv)
 		parse_extended_opts(fs, extended_cmd);
 	if (journal_size || journal_device)
 		add_journal(fs);
-	
+
 	if (U_flag) {
+		int set_csum = 0;
+		dgrp_t i;
+
+		if (sb->s_feature_ro_compat &
+		    EXT4_FEATURE_RO_COMPAT_GDT_CSUM) {
+			/*
+			 * Determine if the block group checksums are
+			 * correct so we know whether or not to set
+			 * them later on.
+			 */
+			for (i = 0; i < fs->group_desc_count; i++)
+				if (!ext2fs_group_desc_csum_verify(fs, i))
+					break;
+			if (i >= fs->group_desc_count)
+				set_csum = 1;
+		}
 		if ((strcasecmp(new_UUID, "null") == 0) ||
 		    (strcasecmp(new_UUID, "clear") == 0)) {
 			uuid_clear(sb->s_uuid);
@@ -1579,6 +1700,11 @@ int main (int argc, char ** argv)
 			com_err(program_name, 0, _("Invalid UUID format\n"));
 			exit(1);
 		}
+		if (set_csum) {
+			for (i = 0; i < fs->group_desc_count; i++)
+				ext2fs_group_desc_csum_set(fs, i);
+			fs->flags &= ~EXT2_FLAG_SUPER_ONLY;
+		}
 		ext2fs_mark_super_dirty(fs);
 	}
 	if (I_flag) {
@@ -1588,6 +1714,14 @@ int main (int argc, char ** argv)
 				"unmounted.\n"), stderr);
 			exit(1);
 		}
+		if (fs->super->s_feature_incompat &
+		    EXT4_FEATURE_INCOMPAT_FLEX_BG) {
+			fputs(_("Changing the inode size not supported for "
+				"filesystems with the flex_bg\n"
+				"feature enabled.\n"),
+			      stderr);
+			exit(1);
+		}
 		/*
 		 * We want to update group descriptor also
 		 * with the new free inode count
@@ -1595,16 +1729,16 @@ int main (int argc, char ** argv)
 		fs->flags &= ~EXT2_FLAG_SUPER_ONLY;
 		if (resize_inode(fs, new_inode_size)) {
 			fputs(_("Error in resizing the inode size.\n"
-				"Run undoe2fs to undo the "
+				"Run e2undo to undo the "
 				"file system changes. \n"), stderr);
 		} else {
-			printf (_("Setting inode size %lu\n"),
+			printf(_("Setting inode size %lu\n"),
 							new_inode_size);
 		}
 	}
 
 	if (l_flag)
-		list_super (sb);
+		list_super(sb);
 	if (stride_set) {
 		sb->s_raid_stride = stride;
 		ext2fs_mark_super_dirty(fs);
@@ -1616,5 +1750,5 @@ int main (int argc, char ** argv)
 		printf(_("Setting stripe width to %d\n"), stripe_width);
 	}
 	remove_error_table(&et_ext2_error_table);
-	return (ext2fs_close (fs) ? 1 : 0);
+	return (ext2fs_close(fs) ? 1 : 0);
 }
