@@ -176,8 +176,7 @@ errout:
  */
 static void fix_uninit_block_bitmaps(ext2_filsys fs)
 {
-	int		i;
-	blk_t		blk, super_blk, old_desc_blk, new_desc_blk;
+	blk_t		i, blk, super_blk, old_desc_blk, new_desc_blk;
 	int		old_desc_blocks;
 	dgrp_t		g;
 
@@ -232,6 +231,35 @@ static void fix_uninit_block_bitmaps(ext2_filsys fs)
  * --------------------------------------------------------------------
  */
 
+/*
+ * If the group descriptor's bitmap and inode table blocks are valid,
+ * release them in the specified filesystem data structure
+ */
+static void free_gdp_blocks(ext2_filsys fs, struct ext2_group_desc *gdp)
+{
+	blk_t	blk;
+	int	j;
+
+	if (gdp->bg_block_bitmap &&
+	    (gdp->bg_block_bitmap < fs->super->s_blocks_count))
+		ext2fs_block_alloc_stats(fs, gdp->bg_block_bitmap, -1);
+
+	if (gdp->bg_inode_bitmap &&
+	    (gdp->bg_inode_bitmap < fs->super->s_blocks_count))
+		ext2fs_block_alloc_stats(fs, gdp->bg_inode_bitmap, -1);
+
+	if (gdp->bg_inode_table == 0 ||
+	    (gdp->bg_inode_table >= fs->super->s_blocks_count))
+		return;
+
+	for (blk = gdp->bg_inode_table, j = 0;
+	     j < fs->inode_blocks_per_group; j++, blk++) {
+		if (blk >= fs->super->s_blocks_count)
+			break;
+		ext2fs_block_alloc_stats(fs, blk, -1);
+	}
+}
+
 /*
  * This routine is shared by the online and offline resize routines.
  * All of the information which is adjusted in memory is done here.
@@ -375,6 +403,14 @@ retry:
 	 * can exit now.
 	 */
 	if (old_fs->group_desc_count > fs->group_desc_count) {
+		/*
+		 * Check the block groups that we are chopping off
+		 * and free any blocks associated with their metadata
+		 */
+		for (i = fs->group_desc_count;
+		     i < old_fs->group_desc_count; i++) {
+			free_gdp_blocks(fs, &old_fs->group_desc[i]);
+		}
 		retval = 0;
 		goto errout;
 	}
@@ -746,7 +782,7 @@ static errcode_t blocks_to_move(ext2_resize_t rfs)
 		g = ext2fs_group_of_blk(fs, blk);
 		if (EXT2_HAS_RO_COMPAT_FEATURE(fs->super,
 					       EXT4_FEATURE_RO_COMPAT_GDT_CSUM) &&
-		    (fs->group_desc[g].bg_flags & EXT2_BG_BLOCK_UNINIT)) {
+		    (old_fs->group_desc[g].bg_flags & EXT2_BG_BLOCK_UNINIT)) {
 			/*
 			 * The block bitmap is uninitialized, so skip
 			 * to the next block group.
@@ -1375,8 +1411,8 @@ errout:
 struct istruct {
 	ext2_resize_t rfs;
 	errcode_t	err;
-	unsigned long	max_dirs;
-	int		num;
+	unsigned int	max_dirs;
+	unsigned int	num;
 };
 
 static int check_and_change_inodes(ext2_ino_t dir,
@@ -1918,5 +1954,13 @@ blk_t calculate_minimum_resize_size(ext2_filsys fs)
 	blks_needed = (groups-1) * EXT2_BLOCKS_PER_GROUP(fs->super);
 	blks_needed += overhead;
 
+	/*
+	 * We need to reserve a few extra blocks if extents are
+	 * enabled, in case we need to grow the extent tree.  The more
+	 * we shrink the file system, the more space we need.
+	 */
+	if (fs->super->s_feature_incompat & EXT3_FEATURE_INCOMPAT_EXTENTS)
+		blks_needed += (fs->super->s_blocks_count - blks_needed)/500;
+
 	return blks_needed;
 }
